/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2005, 2019                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* APAR    DATE      DESCRIPTION                                     */
/* D14343  20060420	Got CTL-6D9 with command  ZTEST LISH START       */
/* F19211  20090619  Less 4K frame available due to PPA project      */
/* d26352  20180810  Compiler compatibility updates                  */
/* d26048  20190115  XWB support                                     */
/*                                                                   */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Program: QXN0.CPP                                                 */
/* This ecb allocates resources to put it in inputlist               */
/* shutdown. Then it deallocates the resource                        */
/*                                                                   */
/* ARGV[1] - The address of a Table for the driver                   */
/*           configuration.                                          */
/*********************************************************************/
#include "slecb.h"
#include <tpf/c_decb.h>

#define OWNER    'C'
#define OFF_TIMER 1
#define ON_TIMER  1
#define PARTMEG 100
// #define FOURKPARTITION 1000
#define FOURKPARTITION 925                                            /*19211*/
#define FRMS 100
#define MINFRM 5

int delay_timer = 10;
int meg_res = 0;
int meg_part = 0;
int new_maxchilds = 0;
enum t_getfmt format = (enum t_getfmt)(GETCC_TYPE+GETCC_SIZE+GETCC_COMMON);
enum t_getfmt format_4k = (enum t_getfmt)(GETCC_TYPE);

int TOTAL_CMBS = 0;                                                   /*14343*/
TPF_DECB   *decb=(TPF_DECB *)NULL;
DECBC_RC   decb_rc;

struct     _1kblock *blocks = NULL,
	*hblock          = NULL,
	*tempblock       = NULL;

void create_ecb(sysinfo *swb_addr, char *parms);

void allocate_resource(struct struct_task *childlist,
					   sysinfo *ECB_TABLE,
					   int ecb_i,
					   char *ecb_name);


void deallocate_resource(struct struct_task *childlist,
						 sysinfo *ECB_TABLE,
						 int ecb_i,
						 char *ecb_name);


void run_ecb(sysinfo * swb_addr, int ecb_id);

int main(int argc, char *argv[])
{

	/*******************************/
	/* MAXSUSP/CRECLASS test cases */
	/*******************************/
	if (memcmp(&ecbptr()->ebw000, CASE_ID, sizeof(CASE_ID)) == 0)
		doTestCase();


	struct  ev0bk pb;
	char    ecb_name[9];
	int     oldpk;
	struct  struct_task *childlist;
	sysinfo *ECB_TABLE;

	cinfc(CINFC_WRITE, C_PGM);
	oldpk    = ecbp2() -> ce2okey ;
	strcpy(ecb_name,"LISHDJR5");

	ECB_TABLE  = (sysinfo *)atol(argv[1]);
	childlist = ECB_TABLE->ecb;
	lodic_ext((t_lodic_opt)(LODIC_UNMARK),NULL);
	run_ecb(ECB_TABLE, 0);  /* create ECB to print msg either lodic, or INPUTLIST */
	event_handler(&pb, NULL,10);

	if (ECB_TABLE->shutdownlist == INPUTLISTSHUTDOWNLEVEL ||
		(ECB_TABLE->shutdownlist == LODICSHUTDOWNLEVEL &&
		ECB_TABLE->flag_sset == -1))

		allocate_resource(childlist, ECB_TABLE, 0, ecb_name);


	event_handler(&pb, NULL,10); /* hold 30 sec */

	/*************************************************************************/
	/* Since (theoretical) lodic shutdown comes first. Once this ECB is      */
	/* placed in lodic shutdown, it waits until the system goes to inputlist */
	/* shutdown before starts to release resources                           */
	/*************************************************************************/

	if (ECB_TABLE->shutdownlist == LODICSHUTDOWNLEVEL)
	{
		ECB_TABLE->shutdown = YES;
		while (ECB_TABLE && ECB_TABLE->shutdown == YES &&
			ECB_TABLE->request_exit!=YES)
		{


			event_handler(&pb, NULL, 5);

		}
	}


	if ((ECB_TABLE->flag_sset !=-1) &&
		ECB_TABLE->shutdownlist == INPUTLISTSHUTDOWNLEVEL)
	{
		ECB_TABLE->shutdown = YES;
		while (ECB_TABLE && !INPUTLIST_ok() && ECB_TABLE->request_exit!=YES)
		{


			event_handler(&pb, NULL, 5);

		}


		if (ECB_TABLE->request_exit) ECB_TABLE->shutdown=YES;
		if (INPUTLIST_ok() && (ECB_TABLE->shutdownlist == INPUTLISTSHUTDOWNLEVEL))
		{


			if (ECB_TABLE->rra!=ECB)
				run_ecb(ECB_TABLE, 1);  /* create ECB to print msg either lodic, or INPUTLIST */
			ECB_TABLE->shutdown = YES;
			printinfo(INFO_SHD);

			event_handler(&pb, NULL, ECB_TABLE->timer);
		}

	}

	deallocate_resource(childlist, ECB_TABLE, 0, ecb_name);

	/* Force any ecb hanging to get out */
	struct ev0bk wakeup_main_driver;
	wakeup_main_driver.evnpstinf.evnbkc1 = 1;
	memcpy((char *)&wakeup_main_driver.evnbkn, "ecblishd",8);
	postc(&wakeup_main_driver, EVENT_CNT,0);
	ecbp2()->ce2okey = oldpk;
	keyrc();
	exit(0);
	return 0;

} /* END MAIN */


/*****************************************************************/
/* Allocate resource                                             */
/*****************************************************************/
void allocate_resource(struct struct_task *childlist,
					   sysinfo *ECB_TABLE,
					   int ecb_i,
					   char *ecb_name)
{


	struct ev0bk pb, me;
	int    iblock = 0, timernow = 0;
	char name_dummy[9];
	event_handler(&me, "ecblishd", 0);
	int done = 1;
	if (ECB_TABLE->shutdownlist != INPUTLISTSHUTDOWNLEVEL)
		delay_timer = 25;

	if (ECB_TABLE->delay == 1)
		delay_timer = 2; /* over delay */


	switch (ECB_TABLE->rra)
	{

	case ECB: /* Create ECBs */
		ECB_TABLE->ecb_counter=0;
		done = 0;

		for (iblock = 1; iblock < MaxChilds; iblock++)
		{
			if (inputlist_ok(ECB_TABLE, name_dummy)==0) break;

			if (ECB_TABLE->flag_sset != -1)
			{
				if (iblock >= ECB_TABLE->rsize) break;
			}

			if (ECB_TABLE->request_exit==YES) break;

			if (ECB_TABLE->shutdownlist == INPUTLISTSHUTDOWNLEVEL)
			{
				if (INPUTLIST_ok()) break;
			}

			else  if (!check_lodic_values() || INPUTLIST_ok())  break;


			run_ecb(ECB_TABLE, iblock);
			childlist[iblock].running = YES;
			ECB_TABLE->ecb_counter++ ;


			if ((timernow++ % 10)==0)
				event_handler(&pb, NULL, OFF_TIMER+2);

			
		}

		if (ECB_TABLE->flag_sset != -1) break;
		if (ECB_TABLE->flag_sset == -1)
		{
			if (ECB_TABLE->shutdownlist == INPUTLISTSHUTDOWNLEVEL)
				if (INPUTLIST_ok()) break;

			if ((timernow++ % 10)==0)
				event_handler(&pb, NULL, OFF_TIMER+2);

		}

	
		break;

	case SWB:
		for (;;)
		{

			if (inputlist_ok(ECB_TABLE, name_dummy)==0) break;
			if (ECB_TABLE->flag_sset != -1)
			{
				if (iblock >= ECB_TABLE->rsize) break;
			}
			if (ECB_TABLE->request_exit==YES) break;


			if (ECB_TABLE->shutdownlist == INPUTLISTSHUTDOWNLEVEL)
			{
				if (INPUTLIST_ok())
				{
					break;
				}
			}
			else if (!check_lodic_values() || INPUTLIST_ok()) break;

			tempblock  = (struct _1kblock *) CGSWBC(ecb_name);
			if (tempblock)
			{
				if (iblock == 0)
				{

					blocks  = hblock = tempblock;
					blocks->nextblock=NULL;
					iblock++;
				}

				else
				{
					blocks->nextblock = tempblock;
					blocks            = blocks->nextblock;
					blocks->nextblock = NULL;
					iblock++;
				}
			}
			if ((timernow++ % delay_timer)==0)
				event_handler(&pb, NULL,OFF_TIMER); /* Avoid CTL-10 */

		}
		break;

    case XWB:                                                                            //d26048
		for (;;)
		{

			if (inputlist_ok(ECB_TABLE, name_dummy)==0) break;
			if (ECB_TABLE->flag_sset != -1)
			{
				if (iblock >= ECB_TABLE->rsize) break;
			}
			if (ECB_TABLE->request_exit==YES) break;


			if (ECB_TABLE->shutdownlist == INPUTLISTSHUTDOWNLEVEL)
			{
				if (INPUTLIST_ok())
				{
					break;
				}
			}
			else if (!check_lodic_values() || INPUTLIST_ok()) break;

			tempblock  = (struct _1kblock *) CGXWBC(ecb_name);
			if (tempblock)
			{
				if (iblock == 0)
				{

					blocks  = hblock = tempblock;
					blocks->nextblock=NULL;
					iblock++;
				}

				else
				{
					blocks->nextblock = tempblock;
					blocks            = blocks->nextblock;
					blocks->nextblock = NULL;
					iblock++;
				}
			}
			if ((timernow++ % delay_timer)==0)
				event_handler(&pb, NULL,OFF_TIMER); /* Avoid CTL-10 */

		}
		break;                                                                           //d26048
        
	case FRM:
		done = 0;
		if (ECB_TABLE->flag_sset == -1)
			ECB_TABLE->rsize = numbc(LFRAME);
		ECB_TABLE->ecb_counter = 0;
		new_maxchilds = 1;

		if (ECB_TABLE->rsize > FOURKPARTITION)
		{
			new_maxchilds = ECB_TABLE->rsize / FOURKPARTITION;
			if ((ECB_TABLE->rsize % 1000) > 0)
				new_maxchilds ++;

		}

		for (iblock = 1 ; iblock <= new_maxchilds; iblock++)
		{
			done = 1;
			if (inputlist_ok(ECB_TABLE, name_dummy)==0) break;


			if (ECB_TABLE->request_exit==YES) break;

			if (ECB_TABLE->shutdownlist == INPUTLISTSHUTDOWNLEVEL)
			{
				if (INPUTLIST_ok()) break;
			}

			else  if (!check_lodic_values() || INPUTLIST_ok())  break;

			ECB_TABLE->rra = FRM;
			run_ecb(ECB_TABLE, iblock);
			childlist[iblock].running = YES;


			ECB_TABLE->ecb_counter++ ;

			if ((timernow++ % delay_timer)==0)
				event_handler(&pb, NULL, OFF_TIMER);
     	

		}

		if (ECB_TABLE->flag_sset != -1) break;
		if (ECB_TABLE->flag_sset == -1)
		{
			if (ECB_TABLE->shutdownlist == INPUTLISTSHUTDOWNLEVEL)
				if (INPUTLIST_ok()) break;

			if ((timernow++ % delay_timer)==0)
				event_handler(&pb, NULL, OFF_TIMER);

		}


		break;

	case CMB:
		decb = tpf_decb_create(ecb_name, &decb_rc);
		if (decb_rc == DECBC_OK)
		{

			for (;;)
			{
				if (inputlist_ok(ECB_TABLE, name_dummy)==0) break;
				if (ECB_TABLE->flag_sset != -1)
				{
					if (iblock >= ECB_TABLE->rsize) break;
				}

				if (ECB_TABLE->request_exit==YES) break;
				if (ECB_TABLE->shutdownlist == INPUTLISTSHUTDOWNLEVEL)
				{
					if (INPUTLIST_ok()) break;
				}
				else  if (!check_lodic_values() || INPUTLIST_ok()) break;

				if (levtest(decb))
					detac_ext(decb, DETAC_USER_ACPDB);                     /*14343*/

				tempblock  = (struct _1kblock*)getcc( decb, format, L4, ' ' );

				if (tempblock)
				{
               TOTAL_CMBS++;
					tempblock->owner = OWNER;
					if (iblock == 0)
					{

						hblock = tempblock;
						blocks = tempblock;
						blocks->nextblock = NULL;
						iblock++;


					}

					else
					{
						blocks->nextblock = tempblock;
						blocks            = blocks->nextblock;
						blocks->nextblock = NULL;
						iblock++;
					}

				}
				if ((timernow++ % delay_timer)==0)
					event_handler(&pb, NULL,OFF_TIMER+2);

			}


		}
		break;

	case _1MGFRM:


		if (ECB_TABLE->flag_sset == -1)
			ECB_TABLE->rsize = numbc(LFRM1MB) - MINFRM;
		else if (ECB_TABLE->rsize > numbc(LFRM1MB))
			ECB_TABLE->rsize = numbc(LFRM1MB) - MINFRM;

		if (ECB_TABLE->rsize > FRMS)
		{
			meg_part = ECB_TABLE->rsize / FRMS;
			meg_res =  ECB_TABLE->rsize % FRMS;

			if (meg_res > 0) meg_part++;
			else meg_res = PARTMEG;

			ECB_TABLE->rsize = meg_part;
		}
		else meg_res = ECB_TABLE->rsize;

		for (;;)
		{
			if (inputlist_ok(ECB_TABLE, name_dummy)==0) break;
			if (ECB_TABLE->flag_sset != -1)
			{
				if (iblock >= ECB_TABLE->rsize) break;
			}

			if (ECB_TABLE->request_exit==YES) break;
			if (ECB_TABLE->shutdownlist == INPUTLISTSHUTDOWNLEVEL)
			{
				if (INPUTLIST_ok())
				{


					break;
				}
			}

			else if (!check_lodic_values() || INPUTLIST_ok()) break;

			/* D9526  added GSYSC_64BIT */

			/* Fix for defect 10113 */

			if (meg_part == 0) {
				tempblock  = (struct _1kblock *) tpf_gsysc(meg_res, ecb_name,ecb_name,GSYSC_1MB + GSYSC_64BIT );
				if (!tempblock) break;
				tempblock->size = meg_res;
			}
			else
			{


				if ((ECB_TABLE->rsize) == iblock)
				{
					if (meg_res > (numbc(LFRM1MB) -MINFRM))
						meg_res = numbc(LFRM1MB)-MINFRM;


					tempblock  = (struct _1kblock *) tpf_gsysc(meg_res, ecb_name,ecb_name,GSYSC_1MB + GSYSC_64BIT );
					if (!tempblock) break;
					tempblock->size = meg_res;

				}
				else  {

					tempblock  = (struct _1kblock *) tpf_gsysc(FRMS, ecb_name,ecb_name,GSYSC_1MB + GSYSC_64BIT );
					if (!tempblock) break;
					tempblock->size = FRMS;
				}
			}

			/* End for fix 10113 */

			if (tempblock)
			{

				if (iblock == 0)
				{
					// Hold the head of the list
					hblock  = tempblock;

					blocks  = tempblock;
					blocks->nextblock = NULL;
					iblock++;
				}

				else
				{
					blocks->nextblock = tempblock;
					blocks            = blocks->nextblock;
					blocks->nextblock = NULL;
					iblock++;
				}

			}
			if ((timernow++ %  delay_timer)==0)
				event_handler(&pb, NULL,OFF_TIMER+1);
		}

		break;

	default:  /* Nothing to do. Should not get here */
		break;
	}


	/* check if size was set **********************************/
	/* if size was set and we not in inputlist shutdown, exit */
	/**********************************************************/
	if (ECB_TABLE->flag_sset != -1)
	{


		printinfo(CARVING);

		if (INPUTLIST_ok())                         /* D9526 */
			printinfo(INFO_SHD);                    /* D9526 */

		event_handler(&pb, NULL, ECB_TABLE->timer); /* D7209 */
		ECB_TABLE->shutdown = YES;                  /* D7209 */
		ECB_TABLE->request_exit = YES;              /* D7209 */

	}


	if (ECB_TABLE->flag_sset == -1) {

		if (ECB_TABLE->request_exit) ECB_TABLE->shutdown=YES;

		if (ECB_TABLE->shutdownlist == INPUTLISTSHUTDOWNLEVEL)
		{


			if (ECB_TABLE->rra!=ECB)
				run_ecb(ECB_TABLE, 1);  /* create ECB to print msg either lodic, or INPUTLIST */
			ECB_TABLE->shutdown = YES;
			printinfo(INFO_SHD);
			event_handler(&pb, NULL, ECB_TABLE->timer);
		}
	}

}

/*****************************************************************/
/* Deallocate resource                                           */
/*****************************************************************/
void deallocate_resource(struct struct_task *childlist,
						 sysinfo *ECB_TABLE,
						 int ecb_i,
						 char *ecb_name)

{

	int  iblock = 0, timernow = 0;
	int count_mg = 0;
	struct ev0bk pb;


	/***************************/
	/* Release the resource    */
	/***************************/


	event_handler(&pb, NULL,5);

	char msg[128];

	ECB_TABLE->ecb[0].running = NO;
	if (ECB_TABLE->shutdownlist == LODICSHUTDOWNLEVEL)
	{
		if (ECB_TABLE->flag_sset==-1)
		{
			sprintf(msg,"%s%s%s",INFO_RSL,type_msg[ECB_TABLE->rra] ,"\n");
			printinfo(msg);
		}
	}
	else
	{
		sprintf(msg,"%s%s%s",INFO_RSI,type_msg[ECB_TABLE->rra] ,"\n");
		printinfo(msg);
	}

	/* If size was set and this is the lodic table, we do not need to
	remove any resourses */

	if (ECB_TABLE->shutdownlist == LODICSHUTDOWNLEVEL &&
		ECB_TABLE->flag_sset != -1)
	{
		if (ECB_TABLE)
			ECB_TABLE->shutdown = STARTLOADING;

		exit(0); /* we only need the shutdown list */
	}
	switch (ECB_TABLE->rra)
	{




	case ECB:
		for (iblock = 0; iblock <= ECB_TABLE->ecb_counter; iblock++)
		{
			ECB_TABLE->ecb[iblock].running = NO;
			if ((timernow++ % 10)==0) {
				event_handler(&pb, NULL,ON_TIMER+2); /* Avoid CTL-10 */

			}
		}



		break;

	case SWB: blocks = hblock;

		while (blocks)
		{
			tempblock = blocks->nextblock;
			CRSWBC((void*)blocks);
			blocks    = tempblock;
			if ((timernow++ % 20)==0)
				event_handler(&pb, NULL,ON_TIMER);
		}

		break;

	case XWB: blocks = hblock;                                                           //d26048

		while (blocks)
		{
			tempblock = blocks->nextblock;
			CRXWBC((void*)blocks);
			blocks    = tempblock;
			if ((timernow++ % 20)==0)
				event_handler(&pb, NULL,ON_TIMER);
		}

		break;                                                                           //d26048
        
	case FRM:

		for (iblock = 1; iblock <= ECB_TABLE->ecb_counter; iblock++)
		{
			ECB_TABLE->ecb[iblock].running = NO;
			if ((timernow++ % 10)==0)
				event_handler(&pb, NULL,ON_TIMER+2); /* Avoid CTL-10 */
		}


		break;

	case CMB:

		decb = tpf_decb_locate(ecb_name, &decb_rc);

      if (!decb)                                                      /*14343*/
      {                                                               /*14343*/
         break;                                                       /*14343*/
      }
		

		while (TOTAL_CMBS >= 0)                                         /*14343*/
		{                                                               /*14343*/
			   if (!levtest(decb))                                       /*14343*/
              attac_ext(decb, ATTAC_USER_ACPDB );                     /*14343*/
            if (levtest(decb))                                        /*14343*/
			      relcc(decb);                                           /*14343*/
			
            TOTAL_CMBS--;                                             /*14343*/
			if ((timernow++ % 20)==0) {                                  /*14343*/
				event_handler(&pb, NULL, ON_TIMER);                       /*14343*/

			}
		}
      break; /* break from CMB */

	case _1MGFRM:
		blocks = hblock;

		while (blocks)
		{

			tempblock  = blocks->nextblock;
			tpf_rsysc((void*)blocks, blocks->size, ecb_name);

			blocks   = tempblock;
			if ((timernow++ % 20)==0)
				event_handler(&pb, NULL,ON_TIMER);
			count_mg ++;
		}

		break;

	default:
		break;
	}


	if (ECB_TABLE)  {
		ECB_TABLE->shutdown = STARTLOADING;

	}

}

void run_ecb(sysinfo * swb_addr, int ecb_id)
{

	char  timer_msg[32],       /* To print time            */
		type_rs[8];             /* To save resource time at */
	char parm[64];
	time_t start_time;         /* To obtatin current time  */
	struct tm  *t;             /* For time  converstation  */

	swb_addr->ecb_no = ecb_id;
	swb_addr->swb_addr = (long)swb_addr;
	memset(parm,0,64);
	sprintf(parm,"QXN3 %ld %d", (long)swb_addr, ecb_id);

	if (INPUTLIST_ok())
	{
		swb_addr->_argc = 5;
		inputlist_ok(swb_addr, type_rs);
		(void) time(&start_time);
		t = localtime(&start_time);
		sprintf(timer_msg, " %d:%d:%d ",t->tm_hour, t->tm_min, t->tm_sec);
		strcpy(swb_addr->time_in,timer_msg);
		strcpy(swb_addr->r_sd,type_rs); /* Fix none to actual resource */
		create_ecb(swb_addr, parm);

	}
	else
	{
		swb_addr->_argc = 2;
		create_ecb(swb_addr, parm);
	}


}
void create_ecb(sysinfo *swb_addr, char *parms)
{

	char *input_msg_ptr;
	struct swisc_create_input cpulist;

	cpulist.program = "QXN3";
	cpulist.istream = SWISC_IS_BALANCE;


	if (levtest(D0))
		relcc(D0);

	input_msg_ptr =(char *) getcc(D0,GETCC_FILL, L1,0x00 );
	memset(input_msg_ptr, 0, 100);
	strcpy(input_msg_ptr, parms);
	memcpy(ecbptr()->ce1cr0, parms, strlen(parms));


	/* Run this ECB on the input list */
	cpulist.cpu_list            = SWISC_LIST_INPUT;
	cpulist.bypass              = SWISC_BYPASS_NO;
	cpulist.ebw_data            = input_msg_ptr;
	cpulist.ebw_data_length     = strlen(input_msg_ptr);
	cpulist.data_level          = D0;

	if (swb_addr == NULL) exit(0);
	swisc_create(&cpulist);


}


/*******************************/
/* MAXSUSP/CRECLASS test cases */
/*******************************/
void doTestCase()
{
	int             num_toConsume, lodic_shutdown_num, input_shutdown_num, extra_num,
                   isAboveLODIC;

	double          seconds_elapsed;

	DECBC_RC        decb_rc;
	TPF_DECB        *decb=NULL;                       /* Used by CMB, FRM tests  */

	long            save_ptr;                         /* The head of link list   */
	struct ev0bk    pb  = { MAXSUSP_EVENT };

	time_t          start_suspend, backFrom_maxsusp;  /* To obtatin current time */
	struct tm       *time_ptr;                          /* For time  converstation */

	cinfc(CINFC_WRITE, C_PGM);
	ecbptr()->ebcm01 = TRUE;                          /* Default: Verbose = YES  */

	/* Gets parameters passed */
	PARA_PASSED* parameters = (PARA_PASSED*) &ecbptr()->ebw000;
	ecbptr()->ebcm01 = parameters->printmsg;

	/********************************************/
	/* This ECB is created by the ECB test case */
	/* for consuming the ECB resource           */
	/* Sleeps and exits after pre-set time      */
	/********************************************/
	if (parameters->is_ECBcase)
	{
		/* Should not need this:   lodic_ext(LODIC_UNMARK, NULL);  */
		/* The parent ECB was unsuspendable when creating this ECB */

		// Sleep over the test period
		sleep(parameters->minute_maxsusp * 60 + 61);
		exit(0);
	}

	/********************************************/
	/* This ECB is created by the FRM test case */
	/* for consuming the 1000 frames resource   */
	/* Sleeps and exits after pre-set time      */
	/********************************************/                                        //d11295
	if (parameters->is_1000FrameCase)                                                     //d11295
	{
      struct _1kblock *addr=NULL, *tempPtr=NULL;                                         //d11295
      DECBC_RC        decb_rc;                                                           //d11295
      TPF_DECB        *decb=NULL;                                                        //d11295

      int      error = 0;                                                                //d11295
      long     pointer = 0x00;                                                  //d26352

	   decb = tpf_decb_create(CASE_ID, &decb_rc);                                         //d11295
      if (decb_rc != DECBC_OK)  {                                                        //d11295
         prterr("QXN0: FRM test case- failed creating decb, QXN0 exits");                //d11295
         exit(0);                                                                        //d11295
      }

		// Consume the request number of FRM
		for (int index = 0; index < FOURKPARTITION; ++index)                               //d11295
		{
			if (levtest(decb))                                                              //d11295
				detac_ext(decb, DETAC_USER_ACPDB);                                              //d11295

			// Get the Common Block- getcc
			addr = (struct _1kblock*) getcc(decb, format_4k, L4, ' ');                      //d11295

			if (addr)  {                                                                    //d11295
				if (index == 0)                                                              //d11295
				{
					pointer = (long) addr;                                                    //d11295
					tempPtr = addr;                                                           //d11295
				}
				else                                                                         //d11295
				{
					tempPtr = tempPtr->nextblock = addr;                                      //d11295
				}
				tempPtr->nextblock = NULL;                                                   //d11295
			}
			else  {                                                                         //d11295
				++error;                                                                     //d11295
			}
		}
		if (error)   {                                                                     //d11295
			sprintf(msg, "QXN0: FRM test case- %d error occurred in getting FRM", error);   //d11295
			prtmsg(msg);                                                                    //d11295
		}

		// Sleep over the test period
		sleep(parameters->minute_maxsusp * 60 + 61);                                       //d11295
		exit(0);                                                                           //d11295
	}

	/********************************************/
	/* The CMB/FRM test cases need decb support */
	/********************************************/
	if (parameters->resource_type == CMB || parameters->resource_type == FRM)  {

		decb = tpf_decb_create(CASE_ID, &decb_rc);
		if (decb_rc != DECBC_OK)  {
			prterr("QXN0: Failed creating decb, QXN0 exits");

	      postc(&pb, EVENT_CNT, 0);                                                       //d11295
			exit(0);
		}
	}

	/****************************************/
	/* Check the Priority Class of this ECB */
	/* should be the same as the CRECLASS   */
	/****************************************/
	if (ecbp2()->ce2class != parameters->cre_class)  {

		sprintf(msg, "QXN0: ERROR Priority Class of QXN0 %s (should be %s), QXN0 exits",
			priority_class[ecbp2()->ce2class],
			priority_class[parameters->cre_class]);
		prterr(msg);

		postc(&pb, EVENT_CNT, 0);                                                          //d11295
		exit(0);
	}
	else
		/* Will bring the system to LODIC shutdown, unmark suspendable for now */
		lodic_ext(LODIC_UNMARK, NULL);

   lodic_shutdown_num = lodic_num(parameters->resource_type, parameters->cre_class);     //d11295
   input_shutdown_num = inputlist_num(parameters->resource_type);                        //d11295
   /* Take away ten percents extra blocks */
   extra_num          = (int) (lodic_shutdown_num * .1);                                 //d11295

   if (input_shutdown_num > lodic_shutdown_num - extra_num)  {                           //d11295
		sprintf(msg, "The lodic shutdown level(%d minus %d) of %s is lower than the input\n"//d11295
                   "shutdown level(%d); The test case will fail, please lower the avail\n"//d11295
                   "number of %s then try again.",                                                    //d11295
                   lodic_shutdown_num,                                                   //d11295
                   extra_num,                                                            //d11295
                   blockTypes[parameters->resource_type],                                //d11295
                   input_shutdown_num,                                                   //d11295
                   blockTypes[parameters->resource_type]);                               //d11295
		prtmsg(msg);                                                                       //d11295

      // Notify QXN5 that the test has been done
		postc(&pb, EVENT_CNT, 0);                                                          //d11295
		exit(0);                                                                           //d11295
   }

	/*************************************************/
	/* Compute the difference between the number of  */
	/* available blocks and the LODIC shutdown level */
	/*************************************************/
	num_toConsume = available_num(parameters->resource_type) - lodic_shutdown_num;        //d11295

	sprintf(msg, num_toConsume < 0
		? "QXN0: Current available %ss:%d is BELOW the LODIC level:%d"
		" (CLASS %s); Will reduce to %d"
		: "QXN0: Current available %ss:%d is ABOVE the LODIC level:%d"
		" (CLASS %s); Will reduce to %d"
		, blockTypes[parameters->resource_type]
		, available_num(parameters->resource_type)
			, lodic_num(parameters->resource_type, parameters->cre_class)
			, priority_class[parameters->cre_class]
			, (available_num(parameters->resource_type) - num_toConsume - extra_num));      //d11295
			prtmsg(msg);

			/* In the case of CREclass=USRLOD1-4, the initial LODIC */
			/* percentages are 100, num_toConsume becomes negative  */
			if (num_toConsume < 0)
				num_toConsume = 0;

			/**********************************/
			/* Consume extra extra_num blocks */
			/**********************************/
			consumeResource(parameters->resource_type,
				num_toConsume+extra_num,                                                     //d11295
				decb,
				&save_ptr,
				parameters);

			lodic_options = (enum t_lodic_opt)(LODIC_CHECK+parameters->enum_cr_class);

         do                                                                              //d11295
			{                                                                               //d11295
            /* In the case of ECB, need waiting for all ECBs */
            /* created if the number is big (hundreds)       */
            defrc();                                                                     //d11295
            // Check whether the system is below the LODIC shutdown level
            // should be below LODIC shutdown level
            isAboveLODIC = lodic_ext(lodic_options, NULL);                               //d11295
			} while (isAboveLODIC);                                                         //d11295

         // Check the ETPFLODIC_LOWRES errno
			if (errno != ETPFLODIC_LOWRES)  {
            prterr("QXN0: ERROR in errno (not equal ETPFLODIC_LOWRES)");
         }
         else  {
            sprintf(msg, "QXN0: Check ETPFLODIC_LOWRES successfully; "
                    "The system is in LODIC shutdown"
                    " (Current available blocks: %d)"
                    , available_num(parameters->resource_type));
            prtmsg(msg);
			}

			/*********************************************/
			/* Test LODIC SUSPEND,MAXSUSP=1,MAXRTN=label */
			/* macro, get the timestamp of the start     */
			/*********************************************/
			time(&start_suspend);
			time_ptr = localtime(&start_suspend);

			sprintf(msg, "QXN0: %d:%d:%d tests LODIC SUSPEND,MAXSUSP=1,MAXRTN=timeout"
				, time_ptr->tm_hour
				, time_ptr->tm_min
				, time_ptr->tm_sec);
			prtmsg(msg);

			// Set the value of Priority Class of QXN0 at ebx012 */
			memcpy(&ecbptr()->ebx012, &parameters->cre_class, sizeof(parameters->cre_class));

			// Test new parameter MAXRTN of LODIC using MAXSUSP-1
			lodicMacro();

			// Check the timestamp after the ECB gets back control after MAXSUSP entered
			time(&backFrom_maxsusp);
			time_ptr = localtime(&backFrom_maxsusp);

			// Calculate the seconds elapsed
			seconds_elapsed = difftime(backFrom_maxsusp, start_suspend);

			sprintf(msg, "QXN0: %d:%d:%d got unsuspended after %.0f seconds"
				, time_ptr->tm_hour
				, time_ptr->tm_min
				, time_ptr->tm_sec
				, seconds_elapsed);
			prtmsg(msg);


			/**************************************************/
			/* Test lodic_ext((LODIC_SUSPEND+LODIC_MAXSUSP..) */
			/**************************************************/
			time(&start_suspend);
			time_ptr = localtime(&start_suspend);

			sprintf(msg, "QXN0: %d:%d:%d tests lodic_ext(LODIC_MAXSUSP-%d..)"
				, time_ptr->tm_hour
				, time_ptr->tm_min
				, time_ptr->tm_sec
				, parameters->minute_maxsusp);
			prtmsg(msg);

			// Use the CRECLASS as the class parameter
			lodic_options = (enum t_lodic_opt)(LODIC_SUSPEND+LODIC_MAXSUSP_SECS+
				LODIC_HOLD+parameters->enum_cr_class);

			lodic_ext(lodic_options,
				NULL,
				parameters->minute_maxsusp);

			/*************************************/
			/* Check the errno ETPFLODIC_MAXSUSP */
			/*************************************/
			if (errno != ETPFLODIC_MAXSUSP)  {
				prterr("QXN0: ERROR in errno (not equal LODIC_MAXSUSP)");
			}
			else  {

				prtmsg("QXN0: Check ETPFLODIC_MAXSUSP successfully; ");

				// Check the timestamp after the ECB gets back control after MAXSUSP entered
				time(&backFrom_maxsusp);
				time_ptr = localtime(&backFrom_maxsusp);

				// Calculate the seconds elapsed
				seconds_elapsed = difftime(backFrom_maxsusp, start_suspend);

				sprintf(msg, "%d:%d:%d got unsuspended after %.0f seconds"
					, time_ptr->tm_hour
					, time_ptr->tm_min
					, time_ptr->tm_sec
					, seconds_elapsed
					, parameters->minute_maxsusp);
				prtmsg(msg);
			}


			sprintf(msg, "QXN0: Complete the %s test case, QXN0 exits"
				, blockTypes[parameters->resource_type]);

			// This message will always show
			wtopc(msg, WTOPC_PRC, WTOPC_NO_CHAIN, WTOPC_NO_HEADER);


			// Release the block type taken
			if (parameters->resource_type == SWB || parameters->resource_type == _1MGFRM || 
                parameters->resource_type == XWB)                                            //d26048
				releaseResource(parameters->resource_type,
				num_toConsume+extra_num,                                                     //d11295
				decb,
				&save_ptr);

			// Notify QXN5 that the test has been done and the blocks has been (will be) released
			postc(&pb, EVENT_CNT, 0);
			exit(0);
}

/********************************************************************/
/* Consume the resource tested %PERCENT_EXTRA below the LODIC level */
/********************************************************************/
void consumeResource(int           type,
					 int           numToConsume,
					 TPF_DECB      *decb,
					 long          *pointer,
					 PARA_PASSED*  parameter)
{
	int index, error = 0;

	struct _1kblock            *swbPtr=NULL,
                              *addr=NULL,
                              *tempPtr=NULL,
                              *xwbPtr=NULL;                                              //d26048
   struct swisc_create_input  swisc_param;

    switch (type)
	{
	case SWB:

		for (index = 0; index < numToConsume; ++index)
		{
			// Get the System Working Block- CGSWBC()
			swbPtr = (struct _1kblock *) CGSWBC("MAXSUSP_SWB");
			if (swbPtr)  {
				if (index == 0)  {
					*pointer = (long) swbPtr;
					tempPtr = swbPtr;
				}
				else
					tempPtr = tempPtr->nextblock = swbPtr;

				tempPtr->nextblock = NULL;
			}
			else
				++error;
		}

		if (error)   {
			sprintf(msg, "QXN0: %d error occurred in getting SWB", error);
			prtmsg(msg);
		}

		break;

    case XWB:                                                                            //d26048

		for (index = 0; index < numToConsume; ++index)
		{
			// Get the System Working Block- CGXWBC()
			xwbPtr = (struct _1kblock *) CGXWBC("MAXSUSP_XWB");
			if (xwbPtr)  {
				if (index == 0)  {
					*pointer = (long) xwbPtr;
					tempPtr = xwbPtr;
				}
				else
					tempPtr = tempPtr->nextblock = xwbPtr;

				tempPtr->nextblock = NULL;
			}
			else
				++error;
		}

		if (error)   {
			sprintf(msg, "QXN0: %d error occurred in getting XWB", error);               //d26048
			prtmsg(msg);
		}

		break;                                                                           //d26048
        
	case CMB:

		// Consume the request number of CMB
		for (index = 0; index < numToConsume; ++index)
		{
			if (levtest(decb))
				detac_ext(decb, DETAC_NOCHECK);

			// Get the Common Block- getcc
			addr = (struct _1kblock*) getcc(decb, format, L4, ' ');

			if (addr)  {
				if (index == 0)
				{
					*pointer = (long) addr;
					tempPtr = addr;
				}
				else
				{
					tempPtr = tempPtr->nextblock = addr;
				}
				tempPtr->nextblock = NULL;
			}
			else  {
				++error;
			}
		}

		if (error)   {
			sprintf(msg, "QXN0: %d error occurred in getting CMB", error);
			prtmsg(msg);
		}

		break;

		// Get storage from the system heap
	case FRM:

      int num_1000Frames, num_remainFrames;                                              //d11295
      num_1000Frames   = numToConsume / FOURKPARTITION;                                  //d11295
      num_remainFrames = numToConsume % FOURKPARTITION;                                  //d11295

		memset((char *) &swisc_param, 0, sizeof(swisc_param));                             //d11295
		swisc_param.program = "QXN0";                                                      //d11295
		swisc_param.istream = SWISC_IS_BALANCE;                                            //d11295
		swisc_param.cpu_list = SWISC_LIST_READY;                                           //d11295
		swisc_param.bypass = SWISC_BYPASS_NO;                                              //d11295
		swisc_param.ebw_data = parameter;                                                  //d11295
		swisc_param.ebw_data_length = sizeof(PARA_PASSED);                                 //d11295
		swisc_param.data_level = NO_LVL;                                                   //d11295

		strcpy(parameter->case_id, CASE_ID);                                               //d11295
		parameter->is_1000FrameCase = TRUE;                                                //d11295

		// Consume the requested number of ECB                                             //d11295
		for (index = 0; index < num_1000Frames; ++index)                                   //d11295
		{
			swisc_create(&swisc_param);                                                     //d11295
		}

		// Consume the remaining number of FRM
		for (index = 0; index < num_remainFrames; ++index)                                 //d11295
		{
			if (levtest(decb))
				detac_ext(decb, DETAC_NOCHECK);

			// Get the Common Block- getcc
			addr = (struct _1kblock*) getcc(decb, format_4k, L4, ' ');

			if (addr)  {
				if (index == 0)
				{
					*pointer = (long) addr;
					tempPtr = addr;
				}
				else
				{
					tempPtr = tempPtr->nextblock = addr;
				}
				tempPtr->nextblock = NULL;
			}
			else  {
				++error;
			}
		}

		if (error)   {
			sprintf(msg, "QXN0: %d error occurred in getting FRM", error);
			prtmsg(msg);
		}

		break;

	case _1MGFRM:

		for (index = 0; index < numToConsume; ++index)
		{
			// Get 1 Meg. frame
			/* D9526  added GSYSC_64BIT */
			addr = (struct _1kblock*) tpf_gsysc(NUM_1MEG, FRM1MB_TOKEN, NULL, GSYSC_1MB + GSYSC_64BIT);

			if (addr)  {
				if (index == 0)
				{
					*pointer = (long) addr;
					tempPtr = addr;
				}
				else
				{
					tempPtr = tempPtr->nextblock = addr;
				}
				tempPtr->nextblock = NULL;
			}
			else  {
				++error;
			}
		}

		if (error)   {
			sprintf(msg, "QXN0: %d error occurred in getting 1 Meg. frame", error);
			prtmsg(msg);
		}

		break;

	case ECB:

		memset((char *) &swisc_param, 0, sizeof(swisc_param));
		swisc_param.program = "QXN0";
		swisc_param.istream = SWISC_IS_BALANCE;
		swisc_param.cpu_list = SWISC_LIST_READY;
		swisc_param.bypass = SWISC_BYPASS_NO;
		swisc_param.ebw_data = parameter;
		swisc_param.ebw_data_length = sizeof(PARA_PASSED);
		swisc_param.data_level = NO_LVL;

		strcpy(parameter->case_id, CASE_ID);
		parameter->is_ECBcase = TRUE;

		// Consume the requested number of ECB
		for (index = 0; index < numToConsume; ++index)
		{
			swisc_create(&swisc_param);
		}
		break;

	}
	return;
}

/* Bring the requrested resource below the LODIC level */
void releaseResource(int       resource_type,
					 int       numToConsume,
					 TPF_DECB  *decb,
					 long      *pointer)
{
	int index, rc, error = 0;

	struct _1kblock  *addr, *swbPtr, *tempPtr, *xwbPtr;

	switch (resource_type)
	{
	case SWB:

		swbPtr = (struct _1kblock *) *pointer;

		for (index = 0; index < numToConsume; ++index)
		{
			if (swbPtr)  {

				tempPtr = swbPtr->nextblock;

				// Release a SWB
				CRSWBC((void*)swbPtr);
				swbPtr = tempPtr;
			}
			else   {
				sprintf(msg, "QXN0: Error NULL address (%dth SWB)\n"
					, index+1);
				prtmsg(msg);
			}
		}
		break;
        
   case XWB:                                                                             //d26048

		xwbPtr = (struct _1kblock *) *pointer;

		for (index = 0; index < numToConsume; ++index)
		{
			if (xwbPtr)  {

				tempPtr = xwbPtr->nextblock;

				// Release a XWB
				CRXWBC((void*)xwbPtr);
				xwbPtr = tempPtr;
			}
			else   {
				sprintf(msg, "QXN0: Error NULL address (%dth XWB)\n"
					, index+1);
				prtmsg(msg);
			}
		}
		break;                                                                           //d26048

		/*      case FRM  case CMB: EXITC will do the job (release the blocks)
		addr = (struct _1kblock*) *pointer;
		for (index = 0; index < numToConsume; ++index)
		{
		if (addr)  {
		tempPtr = addr->nextblock;
		decb->cbrw.cb_ptr  = addr;
		decb->cbrw.cb_type = L4;
		relcc(decb);
		addr = tempPtr;
		}
		}
		break;   */

	case _1MGFRM:

		addr = (struct _1kblock*) *pointer;

		for (index = 0; index < numToConsume; ++index)
		{
			if (addr)  {

				tempPtr = addr->nextblock;

				// Release a 1 Meg. frame
				rc = tpf_rsysc(addr, NUM_1MEG, FRM1MB_TOKEN);
				addr = tempPtr;
				if (rc == RSYSC_ERROR)
					++error;
			}
			else   {
				sprintf(msg, "QXN0: Error NULL address (%dth FRM1MB)\n"
					, index+1);
				prtmsg(msg);
			}
		}

		if (error)  {
			sprintf(msg, "QXN0: FRM1MB- %d error in releasing 1 Meg. frame", error);
			prtmsg(msg);
		}
		break;

	} // switch
	return;
}
