/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2005, 2019                           */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*  DEFECT  DATE      DESCRIPTION                                    */
/*  d26352  20180810  Compiler compatibility updates                 */
/*  d26048  20190115  XWB support                                    */
/*                                                                   */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* qxn1.cpp   This segment has the API to handle the computation for */
/*            the OWNER APIs.                                        */
/*                                                                   */
/*********************************************************************/
#include <slecb.h>                                                              //d26352

#define  OWNER    'C'
#include <tpf/i_prse.h>
#include <tpf/tpfapi.h>
#include <tpf/sysapi.h>

/* For formatting my set */
struct   format_myset {
   char  hi[9];
   char  mid[9];
   char  low[17];
   int   count;
   int   total_heap;
   int   heap64;
   int   heap31;
};

#define MSG_HEADER_PFX "LISHD"

struct wtopc_header header = {
        MSG_HEADER_PFX,
        0,
        WTOPC_ERROR,
        WTOPC_SUBSYS_TIME
        };

#define WAIT_TIMER 1
char event_name[9];


#define INPUT_TIMER 15
#define _EOM        0x4E

extern "C" {

   struct ev0bk me, pb;
   TPF_DECB   *decb=(TPF_DECB *)NULL;
   DECBC_RC   decb_rc;
   enum t_getfmt format    = (enum t_getfmt)(GETCC_TYPE+GETCC_SIZE+GETCC_COMMON);
   enum t_getfmt format1mb = (enum t_getfmt)(GETCC_TYPE+GETCC_SIZE);
   enum t_getfmt format4k  = (enum t_getfmt)(GETCC_FILL);
   extern long OWN(void *p); /*** ASM FUNCTION: owner_drv.asm ****/
   struct owner *remote_owner = NULL, *t = NULL;

long call_owner(struct parameters *p)
{
   long addr;
   addr = OWN(p);
   return addr;
}


long TPF_OWNERC (t_iowner **area, t_iowner_name *thisname, enum t_iownerblk blktype,int high)
{
   long rc;

   parameters p;

   memset((void *)&p,'\0',sizeof(struct parameters));
   p.p1 = (void *)thisname;
   p.api = 6;

   if ((int)blktype == 1)
      p.p2 = (long)SWB;
      
   if ((int)blktype == 8)                                                                //d26048
      p.p2 = (long)XWB;                                                                  //d26048

   if ((int)blktype == 4)
      p.p2 = (long)FRM;

   if ((int)blktype == 3)
      p.p2 = (long)CMB;

   if ((int)blktype == 5)
      p.p2 = (long)_1MGFRM;

   if ((int)blktype == 7)
      p.p2 = (long)SYSHEAP;

   if (remote_owner->api[0] == 'M')
   {
      rc = call_owner(&p);
      area = (t_iowner **)rc;
   }

   else  rc = tpf_ownerc(area,thisname,blktype,0); /* Get all */
   return rc;
}
void myname(t_iowner_name * ownername, struct owner *oname, int rs)
{
   int RS = rs;

   if (oname->def == 'N')
      RS = ECB;

   memcpy(ownername->high, oname->hile[RS],8);
   memcpy(ownername->mid, oname->mile[RS],8);
   memcpy(ownername->low, oname->lole[RS],16);



}


/*************************************************************************************************/
/** GET RESOURCES. The below functions will get resources according to the users input. It will **/
/*  either get resources thru the use of the c functions or the macros                          **/
/*************************************************************************************************/

int myGet_GSWBC(struct owner *ownerinfo)
{


  int  count, i, iblock = 0;
  char ownername[33];
  struct _1kblock* tempblock=NULL,* block=NULL;
  count  = ownerinfo->size[SWB];
  int rn = SWB;
  memset(ownername, 0x00,32);

  if (ownerinfo->def == 'N')
     rn = 2;

   memset(ownername,'\0',33);
   memcpy(ownername, ownerinfo->hile[rn],8);
   memcpy(ownername+8, ownerinfo->mile[rn],8);
   memcpy(ownername+16, ownerinfo->lole[rn],16);

  for (i = 0; i < count; i++)
  {
     struct parameters p;
     memset((void *)&p,'\0',sizeof(struct parameters));
     p.api = 1L;
     p.p1 = (char *)ownername;

     if (ownerinfo->api[SWB] == 'M')
     {
        long adr = call_owner(&p);
        if (adr > 0L)
          tempblock = (struct _1kblock*)adr;
      }
      else
        tempblock  = (struct _1kblock*)CGSWBC(ownername);


      if (tempblock)
      {
        if (iblock == 0)
        {
             block = swb_blocks = tempblock;
             block->nextblock = NULL;
			    iblock++;
         }
         else
         {
              block->nextblock = tempblock;
              block            = block->nextblock;
              block->nextblock = NULL;
              iblock++;
          }

       }

       event_handler(&pb, NULL,WAIT_TIMER); /* Avoid CTL-10 */
      }
   return 0;

   } // End myGet_GSWBC()
   

int myGet_GXWBC(struct owner *ownerinfo)                                                 //d26048
{

  int  count, i, iblock = 0;
  char ownername[33];
  struct _1kblock* tempblock=NULL,* block=NULL;
  count  = ownerinfo->size[XWB];
  int rn = XWB;
  memset(ownername, 0x00,32);

  if (ownerinfo->def == 'N')
     rn = 2;

   memset(ownername,'\0',33);
   memcpy(ownername, ownerinfo->hile[rn],8);
   memcpy(ownername+8, ownerinfo->mile[rn],8);
   memcpy(ownername+16, ownerinfo->lole[rn],16);

  for (i = 0; i < count; i++)
  {
     struct parameters p;
     memset((void *)&p,'\0',sizeof(struct parameters));
     p.api = 8L;                                                                         //d26048
     p.p1 = (char *)ownername;

     if (ownerinfo->api[XWB] == 'M')
     {
        long adr = call_owner(&p);
        if (adr > 0L)
          tempblock = (struct _1kblock*)adr;
      }
      else
        tempblock  = (struct _1kblock*)CGXWBC(ownername);


      if (tempblock)
      {
        if (iblock == 0)
        {
             block = swb_blocks = tempblock;
             block->nextblock = NULL;
			    iblock++;
         }
         else
         {
              block->nextblock = tempblock;
              block            = block->nextblock;
              block->nextblock = NULL;
              iblock++;
          }

       }

       event_handler(&pb, NULL,WAIT_TIMER); /* Avoid CTL-10 */
      }
   return 0;                                                                             //d26048

   } // End myGet_GXWBC()          

int  myGet_4kFRM(struct owner *ownerinfo)
{
   int rc, count, i, iblock=0;
   char ownername[33];
   struct _1kblock* tempblock=NULL,* block=NULL;
   count = ownerinfo->size[FRM];
   memset((void *)ownername, '\0',33);

   memcpy(ownername, ownerinfo->hile[FRM],8);
   memcpy(ownername+8, ownerinfo->mile[FRM],8);
   memcpy(ownername+16, ownerinfo->lole[FRM],16);


   struct parameters p;
   memset((void *)&p, '\0',sizeof(struct parameters));

   p.p1 = (void *)ownername;

   if (ownerinfo->def == 'Y') {
      if (ownerinfo->api[FRM] == 'M')
      {

         long adr;
         p.api = 2L;
         adr = call_owner(&p);
       }

       else
       {


          rc = tpf_eownrc(TPF_SET_EOWNR,ownername);
          if (rc != TPF_EOWNRC_OK){
             printf("Problem setting the name for FRM \n");
             exit(0);
           }
        }
      }

      for (i = 0; i < count; i++)
      {
         if (levtest(DA))
           detac_ext(DA, DETAC_NOCHECK);

         if (ownerinfo->api[FRM] == 'M')
         {

            long adr;
            p.api = 3L;
            adr = call_owner(&p);
          }

          else     getcc( DA, format4k, L4, ' ' );

          tempblock =(struct _1kblock*) ecbptr()->ce1cra;
          ecbptr()->ebcida = 0x10;
          if (tempblock)
          {
            tempblock->owner = '4';
            if (iblock == 0)
            {
               block = frm_blocks = tempblock;
               block->nextblock = NULL;
					iblock++;
              }
              else
              {
                 block->nextblock = tempblock;
                 block            = block->nextblock;
                 block->nextblock = NULL;
                 iblock++;
                }
              } // end for if tempblock
      		  event_handler(&pb, NULL,WAIT_TIMER); /* Avoid CTL-10 */

          }// End For loop
       return 0;
    } //myGet_4KFM


 int  myGet_ECB(struct owner *ownerinfo)
    {

          char ownername[33];
          int rc;
          memset((void *)ownername, '\0',33);
          memcpy(ownername, ownerinfo->hile[ECB],8);
          memcpy(ownername+8, ownerinfo->mile[ECB],8);
          memcpy(ownername+16, ownerinfo->lole[ECB],16);


         struct parameters p;
         memset((void *)&p, '\0',sizeof(struct parameters));
         p.api = 2L;
         p.p1 = (void *)ownername;

         // First set the name of the ECB
         if (ownerinfo->api[FRM] == 'M')
         {

              long adr;
              p.api = 2L;
              adr = call_owner(&p);
          }

          else
          {
              rc = tpf_eownrc(TPF_SET_EOWNR,ownername);
              if (rc != TPF_EOWNRC_OK){
                    printf("Problem renaming ECB\n");
                    exit(0);
              }
           }

        return 0;
  } //  myGet_ECB


   int myGet_SYSHEAP(struct owner *ownerinfo)
   {

      int  count;
      long size;
      struct parameters p;
      char *ecb_name = "MSYSHEAP";
      char ownername[33];
      int rn = SYSHEAP;
      memset(ownername, '\0',33);
      if (ownerinfo->def == 'N')
         rn = ECB;

      count = ownerinfo->size[ SYSHEAP];
      memcpy(ownername, ownerinfo->hile[rn],8);
      memcpy(ownername+8, ownerinfo->mile[rn],8);
      memcpy(ownername+16, ownerinfo->lole[rn],16);

      size = (long)count;
      memset((void *)&p,'\0',sizeof(struct parameters));
      p.api = 0L;
      p.p1 = (void *)ownername;
      p.p2 = size;

      if (ownerinfo->api[ SYSHEAP] == 'M')
       {
          long adr;
          adr = call_owner(&p);
          sh_blocks = (struct _1kblock *)adr;

       }
       else
          sh_blocks = (struct _1kblock *)tpf_gsysc(count, ecb_name,ownername, GSYSC_64BIT );


   return 0;

   } // End myGet_4KFRM()



   int myGet_1MBFRM(struct owner *ownerinfo)
   {


      int  count;
      long size;
      char *ecb_name = "ABCDECF";
      struct parameters p;
      int rn=0;
      char ownername[33];
      count = ownerinfo->size[ _1MGFRM];
      memset(ownername,'\0',33);

      if (ownerinfo->def == 'N')
         rn = ECB;

       memcpy(ownername, ownerinfo->hile[rn],8);
       memcpy(ownername+8, ownerinfo->mile[rn],8);
       memcpy(ownername+16, ownerinfo->lole[rn],16);

      size = (long)count;
      memset((void *)&p,'\0',sizeof(struct parameters));
      p.api = 5L;
      p.p1 = (void *)ownername;
      p.p2 = size;

      if (ownerinfo->api[ _1MGFRM] == 'M')
      {
        long adr;
        adr = call_owner(&p);
        frm1mb_blocks = (struct _1kblock *)adr;
      }
      else frm1mb_blocks = (struct _1kblock *)tpf_gsysc(count, ecb_name,ownername, GSYSC_1MB );

   return 0;
   } // End myGet_1MBFRM()


   int myGet_CMB(struct owner *ownerinfo)
   {
      int rc, count, i, iblock=0;
      char ownername[33];
      struct _1kblock* tempblock=NULL,* block=NULL;
      struct parameters p;
      memset((void *)&p,'\0',sizeof(struct parameters));
      memset(ownername, '\0',33);
      count = ownerinfo->size[CMB];
      if (ownerinfo->def == 'Y')
      {
         memcpy(ownername, ownerinfo->hile[CMB],8);
         memcpy(ownername+8, ownerinfo->mile[CMB],8);
         memcpy(ownername+16, ownerinfo->lole[CMB],16);


         p.api = 4L;
         p.p1 = (void *)ownername;

      if (ownerinfo->api[CMB] == 'M')
      {
          long adr;
          p.api = 2L;
          adr = call_owner(&p);
       }
       else
       {
          rc = tpf_eownrc(TPF_SET_EOWNR,ownername);
          if (rc != TPF_EOWNRC_OK){
             printinfo("Problem setting the name for CMB \n");
             exit(0);
          }

        }
        }

        for (i = 0; i < count; i++)
        {
          if (levtest(DA))
              detac_ext(DA, DETAC_NOCHECK);

           if (ownerinfo->api[CMB] == 'M')
           {
              long adr ;
              p.api = 4L;
              adr = call_owner(&p);
            }
            else
            {
               tempblock  = (struct _1kblock*)getcc(DA, format, L4, ' ' );
            }

            tempblock =(struct _1kblock*) ecbptr()->ce1cra;
            ecbptr()->ebcida = 0x33;

            if (tempblock)
            {
              tempblock->owner = OWNER;
              if (iblock == 0)
              {
                 block=cmb_blocks = tempblock;
                 block->nextblock = NULL;
					  iblock++;
               }

             else
             {
                block->nextblock = tempblock;
                block            = block->nextblock;
                block->nextblock = NULL;
                iblock++;
              }
             }
      		 event_handler(&pb, NULL,WAIT_TIMER); /* Avoid CTL-10 */

      }
						

    return 0;
    } //myGet_CMB


  int myRelease_GSWB(int limit)
  {
     int iblocks;
     struct _1kblock * blocks, * tempblock;
     blocks = swb_blocks;

     iblocks =  remote_owner->size[SWB]; 			
     if (limit < iblocks) iblocks = limit;
     if (iblocks < 0) return 0;
     while (iblocks--)
     {
        if (blocks)
        {
            tempblock = blocks->nextblock;
            CRSWBC((void*)blocks);
            blocks    = tempblock;
            swb_blocks = blocks;
        } else {swb_blocks = NULL; return 0;}
        event_handler(&pb, NULL,WAIT_TIMER+7);
     }
  return 0;
  }

int myRelease_GXWB(int limit)                                                            //d26048
  {
     int iblocks;
     struct _1kblock * blocks, * tempblock;
     blocks = xwb_blocks;

     iblocks =  remote_owner->size[XWB]; 			
     if (limit < iblocks) iblocks = limit;
     if (iblocks < 0) return 0;
     while (iblocks--)
     {
        if (blocks)
        {
            tempblock = blocks->nextblock;
            CRXWBC((void*)blocks);
            blocks    = tempblock;
            xwb_blocks = blocks;
        } else {xwb_blocks = NULL; return 0;}
        event_handler(&pb, NULL,WAIT_TIMER+7);
     }
  return 0;                                                                              //d26048
  }
  
  int myRelease_CMB(int limit)
  {
         struct _1kblock * blocks,*tempblock;

         int iblocks;

         blocks = cmb_blocks;
			tempblock = blocks;

         iblocks =  remote_owner->size[CMB];
         if (limit < iblocks) iblocks = limit;
         if (iblocks < 0) return 0;

         while (iblocks--)
         {

                 if (tempblock)
                 {
                     if (blocks->owner == OWNER) {

                        tempblock           = blocks->nextblock;

                        if (levtest(DA))
                           detac(DA);

                        ecbptr()->ce1cra = blocks;
                        ecbptr()->ebcida = 0x33;
                        attac(DA);

                        relcc(DA);
                        blocks              = tempblock;
                        cmb_blocks          = blocks;
                     }
                 }
                 else  {cmb_blocks=NULL; return 0;}

                event_handler(&pb, NULL, WAIT_TIMER);
          }



  return 0;
  }


  int myRelease_1MBFRM(struct owner *l)
  {
    //char *ecb_name = "abcdecf";
    char *ecb_name = "ABCDECF";

    if (remote_owner->size[ _1MGFRM] >= l->size[ _1MGFRM])
        tpf_rsysc((void *)frm1mb_blocks, l->size[ _1MGFRM], ecb_name);
    else {
            int k;
            tpf_rsysc((void *)frm1mb_blocks, l->size[ _1MGFRM], ecb_name);
            k =  l->size[ _1MGFRM] -  remote_owner->size[ _1MGFRM];
            l->size[ _1MGFRM] = k;
            myGet_1MBFRM(l);
    }

    return 0;
  }


  int myRelease_SYSHEAP(struct owner *l)
  {

    char *ecb_name = "MSYSHEAP";
    if (remote_owner->size[SYSHEAP] >= l->size[ SYSHEAP])
        tpf_rsysc((void *)sh_blocks, l->size[ SYSHEAP], ecb_name);
    else {
            int k;
            tpf_rsysc((void *)sh_blocks, l->size[ SYSHEAP], ecb_name);
            k =  l->size[ SYSHEAP] -  remote_owner->size[ SYSHEAP];
            l->size[ SYSHEAP] = k;
            myGet_SYSHEAP(l);
    }

    return 0;
  }


  int myRelease_4KFRM(int limit)
  {
         struct _1kblock * blocks,*tempblock;
         int iblocks;

         blocks = frm_blocks;
			tempblock = blocks;

         iblocks =  remote_owner->size[FRM];
         if (limit < iblocks) iblocks = limit;
         if (iblocks < 0) return 0;

         while (iblocks--)
         {

                 if (tempblock->nextblock)
                 {
                     if (blocks->owner == '4') {

                        tempblock           = blocks->nextblock;
                        if (levtest(DA))
                           detac(DA);
                        ecbptr()->ce1cra = blocks;
                        ecbptr()->ebcida = 0x10;
                        attac(DA);
                        relcc(DA);
                        blocks              = tempblock;
                        frm_blocks          = blocks;

                     }
                 }
                 else {     frm_blocks=NULL; return 0;
                      }
                event_handler(&pb, NULL, WAIT_TIMER);
          }

  return 0;
  }

/*************************************************************************************************/
/** RELEASE RESOURCES. The below functions will get resources according to the users input. It will **/
/*  either get resources thru the use of the c functions or the macros                          **/
/*************************************************************************************************/


    void STOP_EVENT(char *ev_name)
   {
      struct ev0bk wakeup_call;
       wakeup_call.evnpstinf.evnbkc1 = 1;
       memcpy((char *)&wakeup_call.evnbkn,ev_name,8);

       /*****************************************/
       /* Post an event to wake up main driver */
       /****************************************/
       postc(&wakeup_call, EVENT_CNT,0);

   }


/******************************************/
/** WE HAVE MAIN AT THE BOTTOM ************/
/******************************************/


void init(struct owner *ownerinfo)

{
   int i;

   struct ev0bk me, pb;
   char s[80];
  sprintf(s, "%s", "START ALLOCATING RESOURCE(S)");
  if (ownerinfo->idname==0)
      wtopc_text(s);



  if  (ownerinfo->blktype[ECB]==ECB)
     myGet_ECB(ownerinfo);

   for (i=0; i < TOTAL_OP; i++)
   {
      if (remote_owner->blktype[i] == CMB)
          myGet_CMB(ownerinfo);


      else if (remote_owner->blktype[i] == SWB)
      {
          myGet_GSWBC(ownerinfo);
          event_handler(&pb, NULL, WAIT_TIMER);
      }

      else if (remote_owner->blktype[i] == XWB)                                          //d26048
      {
          myGet_GXWBC(ownerinfo);                                                        //d26048
          event_handler(&pb, NULL, WAIT_TIMER);                                          //d26048
      }
      
      else if (remote_owner->blktype[i] == FRM)
      {
           myGet_4kFRM(ownerinfo);
           event_handler(&pb, NULL, WAIT_TIMER);
      }

      else if (remote_owner->blktype[i] ==  _1MGFRM)
      {
          myGet_1MBFRM(ownerinfo);
          event_handler(&pb, NULL, WAIT_TIMER);
      }

      else if (remote_owner->blktype[i] ==  SYSHEAP)
      {
         myGet_SYSHEAP(ownerinfo);
         event_handler(&pb, NULL, WAIT_TIMER);
      }


   }

    /*************************************************/

     if   (remote_owner->blktype[ECB] ==  ECB)
     {
          myGet_ECB(ownerinfo);
          event_handler(&pb, NULL, WAIT_TIMER);
     }
    /************************************************/


     sprintf(s, "%s", "DONE ALLOCATING RESOURCE(S)");


     if (ownerinfo->idname==0)
     {
        wtopc_text(s);
	     event_handler(&me,QXN1RUNS, 0);
     }

}
void release(struct owner *ownerinfo)
{

   int i, limit;
   struct ev0bk me;
   char s[80];


   sprintf(s, "%s", "START RELEASING RESOURCE(S)");
   if (ownerinfo->idname==0)
      wtopc_text(s);


   for (i=0; i < TOTAL_OP; i++)
   {

      limit = ownerinfo->size[i];
      if (remote_owner->blktype[i] == CMB)
      {
         myRelease_CMB(limit);
         event_handler(&me, NULL, 2);
      }

      else if (remote_owner->blktype[i] == SWB)
      {
          myRelease_GSWB(limit);
          event_handler(&me, NULL, 3);
      }

      else if (remote_owner->blktype[i] == XWB)                                          //d26048
      {
          myRelease_GXWB(limit);                                                         //d26048
          event_handler(&me, NULL, 3);                                                   //d26048
      }
      
      else if (remote_owner->blktype[i] == FRM)
      {
         myRelease_4KFRM(limit);
         event_handler(&me, NULL, 3);
      }

      else if (remote_owner->blktype[i] ==  _1MGFRM)
      {
         myRelease_1MBFRM(ownerinfo);
        event_handler(&me, NULL, 3);
      }

      else if (remote_owner->blktype[i] ==  SYSHEAP)
      {
         myRelease_SYSHEAP(ownerinfo);
         event_handler(&me, NULL, 3);
      }


   }


   sprintf(s, "%s\n", "DONE RELEASING RESOURCE(S)");


   if (ownerinfo->idname==0)
      {
         wtopc_text(s);
	  event_handler(&me,QXN1RUNS, 0);
   }

}


   /***********************************************************/
   /* This will stop all events  and release all the resource */
   /***********************************************************/

   void stop_all(struct owner *l)
   {
      /* If this is an ECB, Release all */

      struct ev0bk me;
      if (l->idname==0) {
         wtopc_text("STOPPING DRIVER");
         memcpy((void *)remote_owner, (void *)l, sizeof(struct owner));
        event_handler(&me, NULL, 10);
      }
      else remote_owner = t;
      release(l);

   }

void getmyset(struct owner *myset)
{

  char *name[7]={"SYSHEAP","SWB",
"ECB", " FRM","CMB","FRM1MB","XWB"};                                                     //d26048

  int rs_name;
  long rc;
  t_iowner_data        *d; /* my data */
  t_iowner             *myownset;
  char ECB_NAME[33];
  t_iowner_name thisname;
  format_myset local_set[TOTAL_OP];
  memset(local_set,'\0', sizeof(format_myset)*TOTAL_OP);
  memset(&thisname, '\0', sizeof(t_iowner_name));
  for (rs_name=0; rs_name < TOTAL_OP; rs_name++)
  {
    if (myset->blktype[rs_name] == CMB)
    {
        myname(&thisname, myset, CMB);

        rc = TPF_OWNERC(&myownset,&thisname,IOWNERBLK_CMB,0);
        if (remote_owner->api[0] == 'M')
         myownset = (t_iowner  *)rc;
        if  (myownset->num)
        {

            d = &myownset->outarea;
            local_set[CMB].count = d->entry.blks.cmb;
            memcpy(local_set[CMB].hi,d->owner.high,8);
            memcpy(local_set[CMB].mid,d->owner.mid,8);
            memcpy(local_set[CMB].low,d->owner.low,16);
            free(myownset);
        }


    }
    else  if (myset->blktype[rs_name] == SYSHEAP)
    {
        myname(&thisname, myset, SYSHEAP);
        rc = TPF_OWNERC(&myownset,&thisname,IOWNERBLK_SYSHEAP,0); /* Get all */
        if (remote_owner->api[0] == 'M')
         myownset = (t_iowner  *)rc;
        if  (myownset->num)
        {
            d = &myownset->outarea;
            local_set[SYSHEAP].total_heap = d->entry.sysh.syshtotal;
            local_set[SYSHEAP].heap31 = d->entry.sysh.sysh31;
            local_set[SYSHEAP].heap64 = d->entry.sysh.sysh64;
            memcpy(local_set[SYSHEAP].hi,d->owner.high, 8);

            memcpy(local_set[SYSHEAP].mid,d->owner.mid, 8);
            memcpy(local_set[SYSHEAP].low,d->owner.low, 16);
            free(myownset);
        }
    }

    else  if (myset->blktype[rs_name] == FRM)
    {
       myname(&thisname, myset, FRM);
       rc = TPF_OWNERC(&myownset,&thisname,IOWNERBLK_FRM,0); /* Get all */
       if (remote_owner->api[0] == 'M')
         myownset = (t_iowner  *)rc;
        if  (myownset->num)
        {
            d = &myownset->outarea;
            local_set[FRM].count = d->entry.blks.frm;
            memcpy(local_set[FRM].hi,d->owner.high, 8);

            memcpy(local_set[FRM].mid,d->owner.mid,8);
            memcpy(local_set[FRM].low,d->owner.low,16);
            free(myownset);
        }

    }

    else  if (myset->blktype[rs_name] == SWB)
    {
        myname(&thisname, myset, SWB);
        rc = TPF_OWNERC(&myownset,&thisname,IOWNERBLK_SWB,0); /* Get all */
        if (remote_owner->api[0] == 'M')
         myownset = (t_iowner  *)rc;

        if  (myownset->num)
        {
            d = &myownset->outarea;
            local_set[SWB].count = d->entry.blks.swb;
            memcpy(local_set[SWB].hi,d->owner.high,8);

            memcpy(local_set[SWB].mid,d->owner.mid,8);
            memcpy(local_set[SWB].low,d->owner.low,16);
            free(myownset);
        }

    }
    
    else  if (myset->blktype[rs_name] == XWB)                                            //d26048
    {
        myname(&thisname, myset, XWB);
        rc = TPF_OWNERC(&myownset,&thisname,IOWNERBLK_XWB,0); /* Get all */
        if (remote_owner->api[0] == 'M')
         myownset = (t_iowner  *)rc;

        if  (myownset->num)
        {
            d = &myownset->outarea;
            local_set[XWB].count = d->entry.blks.xwb;
            memcpy(local_set[XWB].hi,d->owner.high,8);

            memcpy(local_set[XWB].mid,d->owner.mid,8);
            memcpy(local_set[XWB].low,d->owner.low,16);
            free(myownset);
        }

    }                                                                                    //d26048
    
    else  if (myset->blktype[rs_name] == _1MGFRM )
    {
        myname(&thisname, myset, _1MGFRM);
        rc = TPF_OWNERC(&myownset,&thisname,IOWNERBLK_FRM1MB,0); /* Get all */
        if (remote_owner->api[0] == 'M')
         myownset = (t_iowner  *)rc;
        if  (myownset->num)
        {
            d = &myownset->outarea;
            local_set[_1MGFRM].count = d->entry.blks.frm1mb;
            memcpy(local_set[_1MGFRM].hi,d->owner.high,8);

            memcpy(local_set[_1MGFRM].mid,d->owner.mid,8);
            memcpy(local_set[_1MGFRM].low,d->owner.low,16);
            free(myownset);
        }

    }


     else  if (myset->blktype[rs_name] == ECB )
     {
       struct parameters p;
       long adr ;
       if (remote_owner->api[0] == 'M')
       {
            memset((void *)&p,'\0',sizeof(struct parameters));
            memset(ECB_NAME, '\0', 33);
            p.api = 7L;
            p.p1 = (char *)ECB_NAME;
            adr = call_owner(&p);
       } else
         rc = tpf_eownrc(TPF_GET_EOWNR, ECB_NAME);
     }

  }

  int i;


  char s[80];

  sprintf(s, "%s", "START DISPLAY");
  wtopc(s,  WTOPC_RO+WTOPC_PRC, WTOPC_CHAIN, &header);
  for (i=0; i< TOTAL_OP; i++)
  {

    if (myset->blktype[i] != -1)
    {
    if (i== SYSHEAP)
    {
     sprintf (s, "   %s HAS NAME %s%s%s  HAS RESOURCE  HEAP31=%d, HEAP64=%d, TOTAL Heap=%d",
                  name[i], local_set[i].hi,local_set[i].mid,
                  local_set[i].low,local_set[i].heap31,
                  local_set[i].heap64,
                  local_set[i].total_heap);
    wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);

    }
    else  if (i != ECB)
    {
    sprintf (s, "   %s HAS NAME %s%s%s, COUNT %d",name[i],
               local_set[i].hi,local_set[i].mid,
               local_set[i].low,local_set[i].count);

     wtopc(s, WTOPC_RO+WTOPC_PRC, WTOPC_CHAIN, WTOPC_NO_HEADER);
     }
   }
  }

  if  (myset->blktype[ECB] == ECB)
  {

    sprintf(s, "   ECB HAS NAME %s, COUNT 1", ECB_NAME);
    wtopc(s,  WTOPC_RO+WTOPC_PRC, WTOPC_CHAIN, WTOPC_NO_HEADER);
  }
  /* Now we are ready to printf */
  sprintf(s, "%s\n", "END DISPLAY");

wtopc(s,WTOPC_RO+WTOPC_PRC, WTOPC_CHAIN_END, WTOPC_NO_HEADER);

}


/* Mysetfor default */
void getmyset_default(struct owner *tmp_o)
{

  char *name[7]={"SYSHEAP","SWB","ECB", " FRM","CMB","FRM1MB","XWB"};                    //d26048
  int i;
  int rs_name;
  long rc;
  t_iowner_data *d;
  t_iowner  *myownset;
  struct owner *myset = tmp_o;
  t_iowner_name thisname;
  char s[80];
  format_myset local_set[TOTAL_OP];

  memset(local_set,'\0', sizeof(format_myset)*TOTAL_OP);
  memset(&thisname, '\0', sizeof(t_iowner_name));
  sprintf(s, "%s", "START DISPLAY");
  wtopc(s,  WTOPC_RO+WTOPC_PRC, WTOPC_CHAIN, &header);

  for (i=0; i < tmp_o->size[ECB]; i++)
  {

	 char temp_num[12];                                                         //d26352
     sprintf(temp_num, "%d", i);                                                //d26352
     memcpy(&myset->hile[ECB][8-strlen(temp_num)], temp_num, strlen(temp_num)); //d26352
	 
     for (rs_name=0; rs_name < TOTAL_OP; rs_name++)
     {
        if (myset->blktype[rs_name] == CMB)
        {
            myname(&thisname, myset, CMB);
            rc = TPF_OWNERC(&myownset,&thisname,IOWNERBLK_CMB,0);
           if (remote_owner->api[0] == 'M')
              myownset = (t_iowner  *)rc;
           if  (myownset->num)
          {
              d = &myownset->outarea;
              local_set[CMB].count = d->entry.blks.cmb;
              memcpy(local_set[CMB].hi,d->owner.high,8);
              memcpy(local_set[CMB].mid,d->owner.mid,8);
              memcpy(local_set[CMB].low,d->owner.low,16);
              free(myownset);
         }


    }
    else  if (myset->blktype[rs_name] == SYSHEAP)
    {
        myname(&thisname, myset, SYSHEAP);
        rc = TPF_OWNERC(&myownset,&thisname,IOWNERBLK_SYSHEAP,0); /* Get all */
        if (remote_owner->api[0] == 'M')
         myownset = (t_iowner  *)rc;
        if  (myownset->num)
        {
            d = &myownset->outarea;
            local_set[SYSHEAP].total_heap = d->entry.sysh.syshtotal;
            local_set[SYSHEAP].heap31 = d->entry.sysh.sysh31;
            local_set[SYSHEAP].heap64 = d->entry.sysh.sysh64;
            memcpy(local_set[SYSHEAP].hi,d->owner.high, 8);

            memcpy(local_set[SYSHEAP].mid,d->owner.mid, 8);
            memcpy(local_set[SYSHEAP].low,d->owner.low, 16);
            free(myownset);
        }
    }

    else  if (myset->blktype[rs_name] == FRM)
    {
       myname(&thisname, myset, FRM);
       rc = TPF_OWNERC(&myownset,&thisname,IOWNERBLK_FRM,0); /* Get all */
       if (remote_owner->api[0] == 'M')
         myownset = (t_iowner  *)rc;
        if  (myownset->num)
        {
            d = &myownset->outarea;
            local_set[FRM].count = d->entry.blks.frm;
            memcpy(local_set[FRM].hi,d->owner.high, 8);

            memcpy(local_set[FRM].mid,d->owner.mid,8);
            memcpy(local_set[FRM].low,d->owner.low,16);
            free(myownset);
        }

    }

    else  if (myset->blktype[rs_name] == SWB)
    {
        myname(&thisname, myset, SWB);
        rc = TPF_OWNERC(&myownset,&thisname,IOWNERBLK_SWB,0); /* Get all */
        if (remote_owner->api[0] == 'M')
         myownset = (t_iowner  *)rc;

        if  (myownset->num)
        {
            d = &myownset->outarea;
            local_set[SWB].count = d->entry.blks.swb;
            memcpy(local_set[SWB].hi,d->owner.high,8);

            memcpy(local_set[SWB].mid,d->owner.mid,8);
            memcpy(local_set[SWB].low,d->owner.low,16);
            free(myownset);
        }

    }
    
    else  if (myset->blktype[rs_name] == XWB)                                            //d26048
    {
        myname(&thisname, myset, XWB);
        rc = TPF_OWNERC(&myownset,&thisname,IOWNERBLK_XWB,0); /* Get all */
        if (remote_owner->api[0] == 'M')
         myownset = (t_iowner  *)rc;

        if  (myownset->num)
        {
            d = &myownset->outarea;
            local_set[XWB].count = d->entry.blks.xwb;
            memcpy(local_set[XWB].hi,d->owner.high,8);

            memcpy(local_set[XWB].mid,d->owner.mid,8);
            memcpy(local_set[XWB].low,d->owner.low,16);
            free(myownset);
        }

    }                                                                                    //d26048
    
    else  if (myset->blktype[rs_name] == _1MGFRM )
    {
        myname(&thisname, myset, _1MGFRM);
        rc = TPF_OWNERC(&myownset,&thisname,IOWNERBLK_FRM1MB,0); /* Get all */
        if (remote_owner->api[0] == 'M')
         myownset = (t_iowner  *)rc;
        if  (myownset->num)
        {
            d = &myownset->outarea;
            local_set[_1MGFRM].count = d->entry.blks.frm1mb;
            memcpy(local_set[_1MGFRM].hi,d->owner.high,8);

            memcpy(local_set[_1MGFRM].mid,d->owner.mid,8);
            memcpy(local_set[_1MGFRM].low,d->owner.low,16);
            free(myownset);
        }

    }



  }

  int i;





  for (i=0; i< TOTAL_OP; i++)
  {

   local_set[i].low[15] = '\0';
   local_set[i].mid[7] = '\0';
   local_set[i].hi[7] = '\0';
    if (myset->blktype[i] != -1)
    {
      if (i== SYSHEAP)
      {


         sprintf (s, "%s HAS NAME %s%s%s, HEAP31=%d, HEAP64=%d",
                     name[i], local_set[i].hi,local_set[i].mid,
                     local_set[i].low,local_set[i].heap31,
                     local_set[i].heap64);
         wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);

      }
      else  if (i != ECB)
      {
         sprintf (s, "%s HAS NAME %s%s%s, COUNT %d",name[i],
                           local_set[i].hi,local_set[i].mid,
                           local_set[i].low,local_set[i].count);

         wtopc(s, WTOPC_RO+WTOPC_PRC, WTOPC_CHAIN, WTOPC_NO_HEADER);
     }
   }
  }


  }
   sprintf(s, "%s\n", "END DISPLAY");

   wtopc(s,WTOPC_RO+WTOPC_PRC, WTOPC_CHAIN_END, WTOPC_NO_HEADER);

}

int wait_until_stop(struct owner *ownerinfo)
{
   struct ev0bk me;

   while (1)
   {

     /* Do this after event checking */
     if (remote_owner->action == 'S') /* Stop */
     {
        ownerinfo->action  = 'S';
        stop_all(ownerinfo);
        return 1;
     }
     else if (remote_owner->action == 'D')
     {

       if (ownerinfo->idname == 0)
        {
            if (ownerinfo->def == 'N')
               getmyset_default(ownerinfo);
            else

            getmyset(ownerinfo);
            if (ownerinfo->idname == 0)
                remote_owner->action = 'N';


       }
     }
     else if (remote_owner->action == 'R')
     {
         release(ownerinfo);
         if (ownerinfo->idname == 0)
            remote_owner->action = 'N';

     }
      event_handler(&me,NULL, 1); /* AVOID CTL-10*/
   }
   return 0; /*Hopefully, we will never get here */
}

int main(int argc, char *argv[])
{

   int rc;

   struct owner local_owner;
   struct ev0bk wakeup_main_driver, setev;
   header.wtopc_number = 5;
   header.wtopc_letter = WTOPC_INFO;


   cinfc(CINFC_WRITE, C_PGM1);
   remote_owner = (struct owner *) tpf_fsysc(QXN1RUNS, NULL);
   memcpy((void *)&local_owner, (void *)remote_owner, sizeof(struct owner));
   memcpy(event_name, (char *)&local_owner.gsyscname, 8);

   if (remote_owner->idname == 0)
   event_handler(&setev, QXN1RUNS, 0);

   if (local_owner.def == 'N')
   {
       t = (struct owner *) tpf_fsysc(event_name, NULL);
       memcpy((void *)&local_owner, (void *)t, sizeof(struct owner));
   }


   if (local_owner.idname  ==0)
      printinfo("DRIVER STARTED");

   init(&local_owner);
   wait_until_stop(&local_owner);

   wakeup_main_driver.evnpstinf.evnbkc1 = 1;
   memcpy((char *)&wakeup_main_driver.evnbkn, QXN1RUNS,8);
   /*****************************************/
   /* Post an event to wake up main driver */
   /****************************************/

   /* Release the storage we obtain */
   if (local_owner.idname == 0)
   {
      rc = rsysc((void *)remote_owner, 1, QXN1RUNS);
      if (local_owner.def == 'N')
         rc = rsysc((void *)t, 1, event_name);

      postc(&wakeup_main_driver, EVENT_CNT,0);

   }

   else {  rc = rsysc((void *)t, 1, event_name);

        }
        keyrc_okey();

   if (local_owner.idname  ==0)
      wtopc_text("DRIVER COMPLETED. ");

   //exit(0);
   return 0; /* We will never see this. You can remove it if you like */
}

}
