/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2005, 2019                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*  DEFECT  DATE      DESCRIPTION                                    */
/*          20060911  Removed code related to IBMLO as required by   */
/*                    APAR PJ31330 and to fix compiler errors.       */
/*  d26352  20180810  Compiler compatibility updates                 */
/*  d26048  20190115  XWB support                                    */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Program: QXN5.CPP                                                 */
/* Invoke:  ztest tcpp name-QXN5                                     */
/*                                                                   */
/* Purpose: Main driver. This driver will start/stop QXN0.           */
/*********************************************************************/
#include <slecb.h>                                                              //d26352
#include <tpf/i_prse.h>
#include <tpf/tpfapi.h>

/* grammar for the parser */
int RESOURCE_SIZE =  RESOURCE_MAGICNUMBER;
#define MSG_HEADER_PFX "LISH"
struct wtopc_header header = {
	MSG_HEADER_PFX,
		0,
		WTOPC_ERROR,
		WTOPC_SUBSYS_TIME
};


int RSNI = SWB;
int RSNL = CMB;

#define INPUT_TIMER 15
#define _EOM        0x4E
extern "C" {

	void printf_ownhelp();
	void call_createqxn1(struct owner o);
	void displayLishHelpMsg(void);                                            /*f3350*/
	void dispatchTestCase(int, int);                                          /*f3350*/

	extern int OWN(void *p);
	/* Function to run QNX0 */
	int run_QXN0(char *swb_addr, char *swb_addr2);

	/* Function to initialize the ECB_LILST tables (2) */
	int init_ecblist(sysinfo *ecb_list);

	/* For parse feature 1399 */
	int cmd_parser();

	int handle_owners(struct IPRSE_output *p);

	void QXN5()
	{

		/**** PARSER ****/

		char *qxn5_format =
			"{"
			"{START | STOP}"
			"|{START T-d*}"
			"|{START D-Y}"
			"|{START {I-SWB | I-XWB | I-CMB | I-FRM | I-ECB | I-FRM1MB}}"
			"|{START {I-SWB | I-XWB | I-CMB | I-FRM | I-ECB | I-FRM1MB} D-Y}"
			"|{START {I-SWB | I-XWB | I-CMB | I-FRM | I-ECB | I-FRM1MB} T-d*}"
			"|{START {I-SWB | I-XWB | I-CMB | I-FRM | I-ECB | I-FRM1MB} S-d*}"
			"|{START {I-SWB | I-XWB |I-CMB|I-FRM|I-ECB|I-FRM1MB} T-d* S-d*}"
			"|{START {I-SWB | I-XWB |I-CMB|I-FRM|I-ECB|I-FRM1MB} T-d* S-d* D-Y}"
			"|{CREclass-a++++++ {[MAXSusp-d+]} {[CASE-a+++++]} {[(NO)Verbose]} }"  /*f3350*/
			"|{GET {B-ccc | B-ccccccc | B-ccccc} S-d* H-c M-c L-c {API-C|API-M}  \
			[[B-SWB S-d* H-c M-c L-c {API-C|API-M}], \
            [B-XWB S-d* H-c M-c L-c {API-C|API-M}], \
			[B-CMB S-d* H-c M-c L-c {API-C|API-M}],  \
			[B-FRM S-d* H-c M-c L-c {API-C|API-M}],\
			[B-FRMMB S-d* H-c M-c L-c {API-C|API-M}], \
			[B-SYSHEAP S-d* H-c M-c L-c {API-C|API-M}]]}"
			"|{GET B-ECB S-d+ H-c M-c L-c {API-C|API-M}  \
			[[B-SWB S-d*  {API-C|API-M}], \
            [B-XWB S-d*  {API-C|API-M}], \
			[B-CMB S-d*  {API-C|API-M}],  \
			[B-FRM S-d*  {API-C|API-M}],\
			[B-FRMMB S-d*  {API-C|API-M}], \
			[B-SYSHEAP S-d*  {API-C|API-M}]]}"
			"|{RELEASE {{B-ccc | B-ccccccc| B-ccccc} S-d+ }  \
            [[B-XWB S-d* ], \
			[B-SWB S-d* ],  \
			[B-CMB S-d* ],  \
			[B-FRM S-d* ],\
			[B-FRMMB S-d*],\
			[B-SYSHEAP S-d*]]}"
			"|{END}"
			"|{OWNER HELP}"
			"|{TOTAL {API-C|API-M}}"
			"}";                                                                         //d26048


		int rc, tr=-1, tt=-1,ts=-1, delayme = 0;
		struct IPRSE_output *p;
		struct IPRSE_output rslt;

		struct mi0mi *block_ptr;
		char         *input_ptr;
		char         *eom_ptr;

		char msg[132];
		char prog_name[7];

		char *state_ptr;
		state_ptr = (char *)cinfc_fast_ss(CINFC_CMMSTI,ecbptr()->ce1dbi);
		if ((*(state_ptr) == _CC1052))
		{
			printinfo("This driver run on 1052 or higher state\n");
			exit(0);
		}
		memset(prog_name, 0x00, 6);


		block_ptr = (struct mi0mi *)ecbptr()->ce1cr0;
		memcpy(prog_name, block_ptr->mi0acc,4);
		input_ptr = block_ptr->mi0acc+5;
		eom_ptr = strchr(block_ptr->mi0acc,_EOM);
		*eom_ptr = '\0';

		rc = IPRSE_parse(input_ptr, qxn5_format,
			p = &rslt, IPRSE_ALLOC|IPRSE_PRINT, "QXN5");

		if (rc <= 0)
		{

			if (!strcmp(prog_name,"ZOWN"))
				printf_ownhelp();
			else
				displayLishHelpMsg();

			exit(0);
		}

		/**********************/
		/* CRECLASS test case */
		/**********************/
		if (strcmp(p->IPRSE_parameter,"CREclass-a++++++") == 0)   {

			ecbptr()->ebcm01 = TRUE;                  /* Default: Verbose = YES */
			num_maxsusp = MIN_MAXSUSP;                /* Default: MAXSUSP-2     */

			memcpy(child_class, p->IPRSE_value, strlen(p->IPRSE_value));

			if (memcmp(child_class, "LOBATCH", 7) == 0)  {
				child_priority = LOBATCH;              /* passing the flag to QXN0 */
				cr_option = LODIC_CR_LOBATCH;
				cr_class  = LODIC_LOBATCH;             /* passing the info to QXN0 */
			}
			else if (memcmp(child_class, "IBMHI", 5) == 0)  {
				child_priority = IBMHI;
				cr_option = LODIC_CR_IBMHI;
				cr_class  = LODIC_IBMHI;
			}
			else if (memcmp(child_class, "USRLOD1", 7) == 0)  {
				child_priority = USRLOD1;
				cr_option = LODIC_CR_USRLOD1;
				cr_class  = LODIC_USRLOD1;
			}
			else if (memcmp(child_class, "USRLOD2", 7) == 0)  {
				child_priority = USRLOD2;
				cr_option = LODIC_CR_USRLOD2;
				cr_class  = LODIC_USRLOD2;
			}
			else if (memcmp(child_class, "USRLOD3", 7) == 0)  {
				child_priority = USRLOD3;
				cr_option = LODIC_CR_USRLOD3;
				cr_class  = LODIC_USRLOD3;
			}
			else if (memcmp(child_class, "USRLOD4", 7) == 0)  {
				child_priority = USRLOD4;
				cr_option = LODIC_CR_USRLOD4;
				cr_class  = LODIC_USRLOD4;
			}
			else if (memcmp(child_class, "BATCH", 5) == 0)  {
				child_priority = BATCH;
				cr_option = LODIC_CR_BATCH;
				cr_class  = LODIC_BATCH;
			}
			else  {
				prterr("QXN5: Invalid Create Priority Class, QXN5 exits");
				exit(0);
			}

			p = p->IPRSE_next;

			for ( int i = 1; i < rc; ++i, p = p->IPRSE_next )
			{

				if (!strcmp(p->IPRSE_parameter,"MAXSusp-d+"))
					num_maxsusp = atoi(p->IPRSE_value);

				else if (strcmp(p->IPRSE_parameter,"CASE-a+++++") == 0)
					memcpy(block_type, p->IPRSE_value, strlen(p->IPRSE_value));

				else if (strcmp(p->IPRSE_parameter, "(NO)Verbose") == 0 )
					ecbptr()->ebcm01 = *(p->IPRSE_value) == 'Y' ? TRUE : FALSE;
			}

			/* The Priority Class of QXN5 was fixed to LODIC_IBMHI */
			lodic_options = (enum t_lodic_opt)(LODIC_ECBCREATE+LODIC_IBMHI+cr_option);
			isAboveLODIC = lodic_ext(lodic_options, NULL);

			/* Check the Priority Class of this ECB */
			if (ecbp2()->ce2class != IBMHI)  {

				sprintf(msg, "QXN0: ERROR Priority Class setting of QXN5 (%s)"
					, priority_class[ecbp2()->ce2class]);
				prterr(msg);
			}

			/* This ECB will be suspended once give out the control, don't want continuing */
			if (!isAboveLODIC)   {

				sprintf(msg, "QXN5: ERROR below the shutdown levels of CLASS %s, QXN5 exits"
					, priority_class[ecbp2()->ce2class]);
				prterr(msg);
				exit(0);
			}

			/* Dispatch the test case */
			if (memcmp(block_type, "ALL", 3) == 0)  {
				dispatchTestCase(SWB, num_maxsusp);
                dispatchTestCase(XWB, num_maxsusp);                                      //d26048
				dispatchTestCase(FRM, num_maxsusp);
				dispatchTestCase(CMB, num_maxsusp);
				dispatchTestCase(_1MGFRM, num_maxsusp);
				dispatchTestCase(ECB, num_maxsusp);
				prtmsg("QXN5: All test cases completed, Driver exits\n");
			}
			else if (memcmp(block_type, "ECB", 3) == 0)
				dispatchTestCase(ECB, num_maxsusp);
			else if (memcmp(block_type, "CMB", 3) == 0)
				dispatchTestCase(CMB, num_maxsusp);
			else if (memcmp(block_type, "FRM1MB", 6) == 0)
				dispatchTestCase(_1MGFRM, num_maxsusp);
			else if (memcmp(block_type, "SWB", 3) == 0)
				dispatchTestCase(SWB, num_maxsusp);
            else if (memcmp(block_type, "XWB", 3) == 0)                                  //d26048
				dispatchTestCase(XWB, num_maxsusp);                                      //d26048
			else
				dispatchTestCase(FRM, num_maxsusp);          /* Default block type: FRM */

			exit(0);
		}


		if ((!strcmp(p->IPRSE_value,"GET")) ||(!strcmp(p->IPRSE_value,"RELEASE"))
			|| (!strcmp(p->IPRSE_value,"TOTAL")) ||(!strcmp(p->IPRSE_value,"END"))
			|| (!strcmp(p->IPRSE_value,"OWNER")))
		{

			handle_owners(p); exit(0);
		}

		if(!strcmp(p->IPRSE_value,"STOP"))
		{
			printinfo("STOP");
			struct ev0bk wakeup_main_driver;
			wakeup_main_driver.evnpstinf.evnbkc1 = 1;
			memcpy((char *)&wakeup_main_driver.evnbkn,MAINDRIVER,8);

			/*****************************************/
			/* Post an event to wake up main driver */
			/****************************************/
			postc(&wakeup_main_driver, EVENT_CNT,0);
			exit(0);
		}



		if ((rc > 0) && (p->IPRSE_next))
		{

			if (p)
				p = p->IPRSE_next;

			if ((p) && (*(p->IPRSE_parameter)=='D'))
			{
				delayme = 1;
				p = p->IPRSE_next;
			}

			if ((p) && (*(p->IPRSE_parameter)=='I'))
			{

				strcpy(msg, p->IPRSE_value);
				if (!strcmp(msg,"CMB")) tr = CMB;
				else if (!strcmp(msg,"FRM")) tr = FRM;
				else if (!strcmp(msg,"SWB")) tr = SWB;
                else if (!strcmp(msg,"XWB")) tr = XWB;                                   //d26048
				else if (!strcmp(msg,"ECB")) tr = ECB;
				else if (!strcmp(msg,"FRM1MB")) tr = _1MGFRM;
				if (p) p = p->IPRSE_next; /* Defect 7209 */

			}

			if ((p) && (*(p->IPRSE_parameter)=='T')) /*D7209 */
				tt = atoi(p->IPRSE_value);

			if ((p) && (*(p->IPRSE_parameter)=='S')) /*D7209 */
				ts = atoi(p->IPRSE_value);
			else if ((p) && (*(p->IPRSE_parameter)=='D'))
			{
				delayme = 1;
				p = p->IPRSE_next;
			}

			if (p) p = p->IPRSE_next;

			if ((p) && (*(p->IPRSE_parameter)=='T'))
				tt = atoi(p->IPRSE_value);

			else if ((p) && (*(p->IPRSE_parameter)=='S'))
				ts = atoi(p->IPRSE_value);



			if (p) p = p->IPRSE_next;

			if ((p) && (*(p->IPRSE_parameter)=='D'))
				delayme = 1;

			/*
			else if ((p) && (*(p->IPRSE_parameter)=='S'))
			ts = atoi(p->IPRSE_value);
			*/

		} /* Outer if */



		/*********************************** PARSER ********/




		/**************************************************************/
		/* Table to keep track of ECBs bringing the system down using */
		/* LODIC shutdown values                                      */
		/**************************************************************/

		sysinfo *ecb_listL;

		/*************************************************************/
		/* Table to keep track of ECBs bringing the system down using*/
		/*   ZDCLV values                                            */
		/*************************************************************/

		sysinfo *ecb_listI;

		char  *psp1 = "LODICEYE",  *psp2 = "INPUTEYE", swb_addrL[32], swb_addrI[32];

		struct ev0bk pb, me, cb1;

		int    run=1,  oldpk2, iteration = 1;     /* reset iteration */

		/* We need to query to see if this driver is running */

		memcpy((char *)&pb.evnbkn,MAINDRIVER,8);
		pb.evnpstinf.evnbkc1      = 1;
		if (evnqc(&pb, EVENT_CNT) != EVNQC_NFD)
		{
			printinfo(INFO_ECBR);
			exit(0);
		}

		calculate_entries();


		cinfc(CINFC_WRITE, C_PGM);
		oldpk2    = ecbp2() -> ce2okey ;

		ecb_listI = (sysinfo *)gsysc(FRAME_SIZE, psp2);
		ecb_listL = (sysinfo *)gsysc(FRAME_SIZE, psp1);

		/* Check if either/both of the tables were not allocated, exit */
		if ((!ecb_listI) || (!ecb_listL))
		{
			ecbp2()->ce2okey = oldpk2;
			keyrc();
			printinfo(INFO_NHEAP);
			exit(0);
		}

		ecb_listL->shutdownlist = LODICSHUTDOWNLEVEL;
		ecb_listI->shutdownlist = INPUTLISTSHUTDOWNLEVEL;

		ecb_listL->delay = delayme;
		ecb_listI->delay = delayme;
		/* Set flag */

		ecb_listI->flag_rset = -1;
		ecb_listL->flag_rset = -1;


		ecb_listI->flag_sset = -1;
		ecb_listL->flag_sset = -1;

		if (tr > 0)
		{
			ecb_listI->flag_rset = tr;
			ecb_listL->flag_rset = tr;
			ecb_listL->rra = tr;
			ecb_listI->rra = tr;

			RSNI = tr;
			RSNL = tr;
		}


		ecb_listL->timer = INPUT_TIMER;
		ecb_listI->timer = INPUT_TIMER;
		if (tt > 0)
		{
			ecb_listI->flag_tset = tt;
			ecb_listL->flag_tset = tt;
			ecb_listL->timer = tt;
			ecb_listI->timer = tt;
		}



		if (ts > 0)
		{
			ecb_listI->flag_sset = ts;
			ecb_listL->flag_sset = ts;
			ecb_listL->rsize = ts;
			ecb_listI->rsize = ts;
			RESOURCE_SIZE = ts;
			if (ts>100)
			{
				ecb_listL->flag_sset = ts / 100;
				ecb_listL->flag_sset++;

				ecb_listI->flag_sset = ts / 100;
				ecb_listI->flag_sset++;

			}

		}


		printinfo(INFO_DS);
		/************************************************/
		/* Name this event. We will need a way  to send */
		/*  a signal to it to stop this program/driver  */
		/************************************************/
		event_handler(&me, MAINDRIVER, 0);

		/* Conver the address of the inputlist list (table)  */
		/* to string */
		sprintf(swb_addrL, "%ld", (long )ecb_listL); /* JR_GCC */
		init_ecblist(ecb_listL);

		/* Convert the address of the lodic list (table)    */
		/* to string */
		sprintf(swb_addrI, "%ld", (long )ecb_listI); /* JR_GCC */

		init_ecblist(ecb_listI);



		/* Run QXN0 to bring the system below Input List Shutdown*/
		run_QXN0(swb_addrI, swb_addrL);

		/* Run QXN0 to bring the system below lodic level*/
		run_QXN0(swb_addrL, swb_addrL);


		/* Run until QXN1 sends a signal to stop driver */
		ecb_listI->request_exit = NO;
		ecb_listL->request_exit = NO;
		//while (evnqc(&me, EVENT_CNT) == EVNQC_INC)
		while (run)
		{

			if  (evnqc(&me, EVENT_CNT) != EVNQC_INC)  {
				ecb_listL->request_exit = YES;
				ecb_listI->request_exit = YES;

				if ((ecb_listL->shutdown != STARTLOADING))
					ecb_listL->shutdown = NO;


			}
			/* Check to see if the system is below lodic level */
			if (!check_lodic_values())
			{
				/* IF we are not in lodic, print a msg to the cras every 30 sec */
				if (!printed_lodicshutdown && ecb_listI->shutdown != INLSHUTD && ecb_listI->shutdown != STARTLOADING )
				{

					ecb_listI->shutdown = INLSHUTD;
					printinfo(INFO_MSGLI);
					printed_lodicshutdown = 1;

				}

				event_handler(&cb1,NULL, 30);
			} /* end if lodic */


			/* Check inputlist shutdown limit */

			if (ecb_listI->shutdown == YES || ecb_listI->shutdown ==STARTLOADING)
			{



				if ((ecb_listI->shutdown == STARTLOADING) &&  (ecb_listL->shutdown != STARTLOADING))
					ecb_listL->shutdown = NO;


				if ((iteration >= ITERATION_MAGICNUMBER)  && ecb_listI->shutdown == STARTLOADING
					&& ecb_listL->shutdown == STARTLOADING)
				{

					iteration = 1;


					if (ecb_listI->flag_rset == -1) /* No resource flag set */
					{
						RSNL %= XWB; /* To change resource name 1..6 ECB/FRM/SWB/CMB/_1MGFRM/XWB */   //d26048
						RSNL ++;
						RSNI %= XWB; /* To change resource name 1..6 ECB/FRM/SWB/CMB/_1MGFRM/XWB */   //d26048
						RSNI ++;
					}


					/* Initialize value resouce values */
					ecb_listL->rra         = RSNL;  /* Update resource name   */
					ecb_listL->rsize       = get_reqsize(RSNL);/* Blocks  */
					ecb_listL->ecb_counter = 0;     /* counter for ecb        */
					ecb_listI->rra         = RSNI;  /* Update resource name   */
					ecb_listI->rsize       = get_reqsize(RSNI);;/* Blocks  */
					ecb_listI->ecb_counter = 0;     /* counter for ecb        */

					if (ecb_listL->rra == ECB)
						ecb_listL->rsize   = 1;

					if (ecb_listI->rra == ECB)
						ecb_listI->rsize   = 1;

					/* } End default */
					event_handler(&cb1,NULL, 60);

					if ((ecb_listI->request_exit == YES) ||  (ecb_listL->request_exit == YES))
						run=0;
					else {

						init_ecblist(ecb_listL);
						init_ecblist(ecb_listI);
						run_QXN0(swb_addrL, swb_addrL);
						run_QXN0(swb_addrI, swb_addrL);
					}
				} /* end if iteration >= .. */

				else if (ecb_listI->shutdown == STARTLOADING && ecb_listL->shutdown == STARTLOADING)
				{

					if ((ecb_listI->request_exit == YES) ||  (ecb_listL->request_exit == YES))
					{  run=0;

					}
					else {
						iteration++;
						event_handler(&cb1,NULL, 60);
						init_ecblist(ecb_listL);
						init_ecblist(ecb_listI);
						run_QXN0(swb_addrL, swb_addrL);
						run_QXN0(swb_addrI, swb_addrL);
					}
				} /* End out else */


			} /* loading */

			event_handler(&cb1, NULL, 1);


		} /* end --> while evnqc(&me, EVENT_CNT) == EVNQC_INC) .....  */

		/* Ready to finish running. Send a signal to QXN0 */
		ecb_listL->shutdown    = NO;
		ecb_listI->shutdown    = NO;

		event_handler(&cb1, NULL, 15);
		rsysc((void*)ecb_listI, FRAME_SIZE, psp2);
		ecb_listI = NULL;

		rsysc((void*)ecb_listL, FRAME_SIZE, psp1);
		ecb_listL = NULL;

		ecbp2() -> ce2okey =oldpk2 ;
		keyrc();

		printinfo(INFO_DD);


		exit(0);
	}

	int run_QXN0(char *swb_addr, char *swb_addr2)
	{
		char *input_msg_ptr = NULL;
		char buf[127];
		struct swisc_create_input cpulist;


		cpulist.program = "QXN0";

		if (levtest(D0))
			relcc(D0);


		input_msg_ptr =(char *) getcc(D0,GETCC_FILL, L1,0x00 );


		sprintf(buf, "QXN0 %s %s", swb_addr, swb_addr2);

		strcpy(input_msg_ptr, buf);
		memcpy(ecbptr()->ce1cr0, buf, strlen(buf));

		if (input_msg_ptr)
		{
			cpulist.istream = SWISC_IS_BALANCE;
			cpulist.cpu_list            = SWISC_LIST_INPUT;
			cpulist.bypass              = SWISC_BYPASS_NO;
			cpulist.ebw_data            = input_msg_ptr;
			cpulist.ebw_data_length     = strlen(input_msg_ptr);
			cpulist.data_level          = D0;
			swisc_create(&cpulist);

		}

		return 0;
	}



	int  init_ecblist(sysinfo *ecb_list)
	{
		int i;
		char msg[128];

		calculate_entries();
		ecb_list->shutdown    = NO;         /* Shutdown status       */
		ecb_list->ecb_counter = 0;          /* counter for ecb       */

		ecb_list->rra         = RSNI;       /* Resource Name         */
		if (ecb_list->shutdownlist == LODICSHUTDOWNLEVEL)
			ecb_list->rra           = RSNL;  /* Resource Name         */

		ecb_list->rsize       = RESOURCE_SIZE;/* Blocks       */



		if (ecb_list->shutdownlist == INPUTLISTSHUTDOWNLEVEL) {
			sprintf(msg, "%s%s%s",INFO_RAI,type_msg[ecb_list->rra],")");
			printinfo(msg);
		}
		else
		{
			if (ecb_list->flag_sset==-1) {
				sprintf(msg, "%s%s%s",INFO_RAL, type_msg[ecb_list->rra],")");
				printinfo(msg);
			}
			printed_lodicshutdown = 0;
		}




		/* Small loop no need to give up control */

		for (i=0; i < MaxChilds; i++)
		{
			ecb_list->ecb[i].list_name = NOLIST;
			ecb_list->ecb[i].running   = NO;


		}

		return 0;
	}

	void printf_ownhelp()
	{
		char s[90];
		sprintf(s,"%s","ZOWN END");
		wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		sprintf(s,"%s","     GET { B-BLKTYPE  S-d* H-c M-c L-c {API-C|API-M} }* ");
		wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		sprintf(s,"%s","     GET B-ECB S-dd H-c M-c L-c { API-C|API-M} {B-BLKTYPE S-d* {API-C|API-M} }*");
		wtopc(s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		sprintf(s,"%s","     RELEASE {B-BLKTYPE S-d*}* ");
		wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		sprintf(s,"%s","     OWNER HELP");
		wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		sprintf(s,"%s","     TOTAL API-C");
		wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		sprintf(s,"%s","     where ");
		wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		sprintf(s,"%s","     BLKTYPE = [CMB, FRM, FRMMB, SYSHEAP, SWB, XWB] ");          //d26048
		wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		sprintf(s,"%s","     H-c = c IS A CHARACTER A THRU Z");
		wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		sprintf(s,"%s","     M-c = c Is A CHARACTER A THRU Z");
		wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		sprintf(s,"%s","     L-c = c Is A CHARACTER A THRU Z");
		wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		sprintf(s,"%s","     API-C TO TEST USING THE C FUNTION VERSION");
		wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		sprintf(s,"%s","     API-M TO TEST USING THE MACRO VERSION");
		wtopc( s, WTOPC_RO+WTOPC_PRC,  WTOPC_CHAIN,  WTOPC_NO_HEADER);
		strcpy(s, "End of help");
		wtopc(s,WTOPC_RO+WTOPC_PRC, WTOPC_CHAIN_END, WTOPC_NO_HEADER);



	}
	int handle_owners(struct IPRSE_output *p)
	{

		struct owner o;
		struct ev0bk pb;
		struct owner  *temp_address;
		int i = 0;
		char def = 'Y';

		memset((void *)&o, '\0', sizeof (struct owner));


		char msg[8], c;


		for (i=0; i < TOTAL_OP; i++)
			o.blktype[i] = -1;

		// if (p)    p = p->IPRSE_next;
		if (!strcmp(p->IPRSE_value,"GET")) { /* If GET */
			o.action = 'G';
			p  = p->IPRSE_next;

			i = 0;

			while (*(p->IPRSE_parameter)=='B')
			{
				/***************/
				/* Get Blktype */
				/***************/
				strcpy(msg, p->IPRSE_value);

				if (!strcmp(msg,"CMB")) {i = o.blktype[CMB] = CMB;}
				else if (!strcmp(msg,"FRM")) {i = o.blktype[FRM] = FRM;}
				else if (!strcmp(msg,"SWB")) {i=o.blktype[SWB]= SWB;}
                else if (!strcmp(msg,"XWB")) {i=o.blktype[XWB]= XWB;}                    //d26048
				else if (!strcmp(msg,"FRMMB")) {i=o.blktype[_1MGFRM] =  _1MGFRM;}
				else if (!strcmp(msg,"SYSHEAP")) {i=o.blktype[SYSHEAP] = SYSHEAP;}
				else if (!strcmp(msg,"ECB")) {i=o.blktype[ECB] = ECB;}
				else {printf_ownhelp() ; exit(0);}
				/*************/
				/* Get size  */
				/*************/



				p  = p->IPRSE_next;

				if  (*(p->IPRSE_parameter)!='S') {
					printf_ownhelp(); exit(0);
				}

				o.size[i] = atoi(p->IPRSE_value);


				/*************/
				/* Get Name  */
				/*************/

				if (def == 'Y')
				{
					p  = p->IPRSE_next;
					/*hi*/
					c = *(p->IPRSE_value);
					memset(o.hile[i],c,8);
					p  = p->IPRSE_next;
					/*mi*/
					c = *(p->IPRSE_value);
					memset(o.mile[i],c,8);
					p  = p->IPRSE_next;
					/*loi*/
					c  = *(p->IPRSE_value);
					memset(o.lole[i],c,16);
					if (i == ECB) def = 'N';
				}
				/* Zero last byte */
				o.hile[i][8] = '\0';
				o.mile[i][8] = '\0';
				o.lole[i][16] = '\0';
				/*************/
				/* Get API   */
				/*************/
				p  = p->IPRSE_next;
				o.api[i]=p->IPRSE_value[0];
				if  (*(p->IPRSE_parameter)!='A') {
					printf_ownhelp(); exit(0);
				}
				p  = p->IPRSE_next;



				if (!p) break;

				else if (*(p->IPRSE_parameter)!='B')
				{
					printf_ownhelp(); exit(0);
				}

			}

		} /* END IF GET */

        else if (!strcmp(p->IPRSE_value,"RELEASE")) { /* If RELEASE */          //d26352

				o.action = 'R';
				p  = p->IPRSE_next;
				while (*(p->IPRSE_parameter)=='B')
				{
					/***************/
					/* Get Blktype */
					/***************/
					strcpy(msg, p->IPRSE_value);

					if (!strcmp(msg,"CMB")) {i = o.blktype[CMB] = CMB;}
					else if (!strcmp(msg,"FRM")) {i = o.blktype[FRM] = FRM;}
					else if (!strcmp(msg,"SWB")) {i=o.blktype[SWB]= SWB;}
                    else if (!strcmp(msg,"XWB")) {i=o.blktype[XWB]= XWB;}                //d26048
					else if (!strcmp(msg,"FRMMB")) {i=o.blktype[_1MGFRM] =  _1MGFRM;}
					else if (!strcmp(msg,"SYSHEAP")) {i=o.blktype[SYSHEAP] = SYSHEAP;}
					else if (!strcmp(msg,"ECB")) {i=o.blktype[ECB] = ECB;}
					else {printf_ownhelp(); exit(0);}
					/*************/
					/* Get size  */
					/*************/
					p  = p->IPRSE_next;
					o.size[i] = atoi(p->IPRSE_value);

					p  = p->IPRSE_next;


					if (!p) break;

				}

				memcpy((char *)&pb.evnbkn,QXN1RUNS,8);
				pb.evnpstinf.evnbkc1      = 1;

				if (evnqc(&pb, EVENT_CNT) == EVNQC_NFD)
				{
					wtopc_text("DRIVER NOT RUNNING");
					exit(0);
				}

			} /* end ELSE  RELEASE */

			else if (!strcmp(p->IPRSE_value,"END"))
			{
				o.action = 'S';
				memcpy((char *)&pb.evnbkn,QXN1RUNS,8);
				pb.evnpstinf.evnbkc1      = 1;

				if (evnqc(&pb, EVENT_CNT) == EVNQC_NFD)
				{
					wtopc_text("DRIVER NOT RUNNING");
					exit(0);
				}

			}


			else if (!strcmp(p->IPRSE_value,"TOTAL"))
			{
				o.action = 'D';

				p  = p->IPRSE_next;
				o.api[0]=p->IPRSE_value[0];
				memcpy((char *)&pb.evnbkn,QXN1RUNS,8);
				pb.evnpstinf.evnbkc1      = 1;

				if (evnqc(&pb, EVENT_CNT) == EVNQC_NFD)
				{
					wtopc_text("DRIVER NOT RUNNING");
					exit(0);
				}
			}
			else if (!strcmp(p->IPRSE_value,"OWNER"))
			{

				/* Anything will get us here..including zown owner yyyyy */
				printf_ownhelp();
				exit(0);
			}
			/* Find out if QXN1 is running */

			memcpy((char *)&pb.evnbkn,QXN1RUNS,8);
			pb.evnpstinf.evnbkc1      = 1;
			cinfc(CINFC_WRITE, C_PGM1);

			temp_address = (struct owner *) tpf_fsysc(QXN1RUNS, NULL);

			if ((evnqc(&pb, EVENT_CNT) != EVNQC_NFD) || (temp_address))
			{


				if (o.action == 'G')
				{
					wtopc_text("DRIVER ALREADY RUNNING");
					exit(0);

				}

				// struct ev0bk wakeup_main_driver;

				memcpy(temp_address, (void *)&o, sizeof(struct owner));
				keyrc_okey();

				/*****************************************/
				/* Post an event to wake up main driver */
				/****************************************/

				exit(0);
			}

			call_createqxn1(o);

			/* First time we call qxn5 for owner */
			/*******************
			cinfc(CINFC_WRITE, C_PGM);
			temp_address = (struct owner *)tpf_gsysc(1, QXN1RUNS,QXN1RUNS,GSYSC_UNIQUE );
			if (temp_address)  {
			memcpy(temp_address, (void *)&o, sizeof(struct owner));
			cremc(0, NULL,QXN1);
			}
			else printf("No address found \n");
			keyrc_okey();
			*****/

			return 0;
	}

	void call_createqxn1(struct owner o)
	{
		struct owner tmp_o, *remote_address, *temp_address;
        char gsyscname[20];                                                     //d26352
		struct ev0bk pb;
		if (o.size[ECB] == 0)
		{
			cinfc(CINFC_WRITE, C_PGM);
			o.def = 'Y'; 
			o.idname = 0;
			memcpy(&o.gsyscname,QXN1RUNS, 8);
			remote_address = (struct owner *)tpf_gsysc(1, QXN1RUNS,QXN1RUNS,GSYSC_UNIQUE );
			if (remote_address)  {
				memcpy(remote_address, (void *)&o, sizeof(struct owner));
				cremc(0, NULL,QXN1);
			}
			keyrc_okey();
		}
		else
		{
			int i;

            char temp_num[12];                                                  //d26352

			remote_address = (struct owner *)tpf_gsysc(1, QXN1RUNS,QXN1RUNS,GSYSC_UNIQUE );
			cinfc(CINFC_WRITE, C_PGM);
			for (i = 0; i < o.size[ECB]; i++)
			{
				memset((void *)&tmp_o, 0x00, sizeof (struct owner));
				memset(gsyscname, 0x00, 9);
                if (i<10)
                    sprintf(gsyscname, "QXN1000%d",i);
                else
                    sprintf(gsyscname, "QXN100%d",i);
                sprintf(temp_num, "%d", i);                                     //d26352
                memcpy(&o.hile[ECB][8-strlen(temp_num)],                        //d26352
                       temp_num, strlen(temp_num));                             //d26352
					   
				memcpy((void *)&tmp_o, (void *)&o,sizeof (struct owner));
				memcpy(&tmp_o.gsyscname,gsyscname, 8);
				tmp_o.idname = i;
				tmp_o.def = 'N';

				/* We need to relay some information to the remote ecn that it is about to be created.
				He can locate this memory by using the name QXN1RUNS */

				//cinfc(CINFC_WRITE, C_PGM);
				if (remote_address)  { /* This memory is known to every ECB */
					memcpy(remote_address, (void *)&tmp_o, sizeof(struct owner));

				}



				temp_address = (struct owner *)tpf_gsysc(1, gsyscname, gsyscname, GSYSC_UNIQUE );
				if (temp_address)  {
					memcpy(temp_address, (void *)&tmp_o, sizeof(struct owner));
					cremc(0, NULL, QXN1);
					event_handler(&pb,NULL, 3);
				}
			}
			keyrc_okey();

		}

	}


	/*******************************/
	/* MAXSUSP/CRECLASS test cases */
	/*******************************/
	void dispatchTestCase(int type, int minutes)
	{
		int            rc, timeout;
		struct ev0bk   eb;
		eb.evnpstinf.evnbkc1 = 1;
		strcpy (eb.evnbkn, MAXSUSP_EVENT);

		PARA_PASSED    parameters;

		// Intialize the values passed to the child ECB
		memset(&parameters, 0, sizeof(PARA_PASSED));
		// parameters.is_ECBcase = FALSE; implied

		strcpy(parameters.case_id, CASE_ID);
		parameters.resource_type  = type;
		parameters.minute_maxsusp = minutes;          /* Default MIN_MAXSUSP(2) */
		parameters.cre_class      = child_priority;   /* Set CRECLASS           */
		parameters.enum_cr_class  = cr_class;
		parameters.printmsg       = ecbptr()->ebcm01; /* Set the verbose option */

		/* Set the time to wait for the signal from QXN0      */
		/* 65 is for The LODIC macro test and extra 5 sencods */
		timeout = num_maxsusp * 60 + 65;

		rc = evntc(&eb, EVENT_CNT, 'Y', timeout, EVNTC_NORM);

		if (rc) {
			prterr("QXN5: evntc() error of Maxsusp- The event name already exists;"
				" QXN5 exits.");
			exit(0);
		}

		// Create an ECB to run the requested test case
		cremc(sizeof(parameters), &parameters, QXN0);

		rc = evnwc(&eb, EVENT_CNT);

		// Didn't get postc() signal from QXN0
		if (rc)
			sprintf(msg, "QXN5: Didn't receive message from QXN0 after %d seconds have elapsed"
			, timeout);
		else
			sprintf(msg, "QXN5: Completed the %s test case", blockTypes[parameters.resource_type]);

		prtmsg(msg);

		return;
	}

	/************************/
	/*     Help Message     */
	/************************/
	void displayLishHelpMsg(void)
	{
		struct wtopc_header  output_header;
		wtopc_insert_header(&output_header,
			"QXN5",
			123,
			'I',
			WTOPC_SYS_TIME);

		wtopc("ZTEST LISH COMMAND FORMAT:"
			"\n ZTEST LISH -\n"
			"  START\n"
			"  START T-n\n"
			"        D-y\n"
			"        I-cmb|frm|frm1mb|swb|xwb|ecb\n"
			"        I-cmb|frm|frm1mb|swb|xwb|ecb T-n",
			WTOPC_EBROUT,
			WTOPC_CHAIN,
			&output_header);                                                             //d26048

		wtopc("                                 D-y\n"
			"                                 S-n\n"
			"  CREclass-c MAXSusp-d CASE-b NOV\n"
			"        c:BATCH|LOBATCH|IBMHI|IBMLO|USRLOD1|USRLOD2|USRLOD3|USRLOD4\n"
			"        b:ALL|cmb|frm|frm1mb|swb|xwb|ecb\n"
			"  STOP\n"
			"  HELP/?\n"
			" END OF HELP MESSAGE",
			WTOPC_EBROUT,
			WTOPC_CHAIN_END,
			WTOPC_NO_HEADER);                                                            //d26048
	}

} /* END C++ WRAPPER */

