/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2005, 2019                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*  DEFECT  DATE      DESCRIPTION                                    */
/*          20060911  Removed code related to IBMLO as required by   */
/*                    APAR PJ31330 and to fix compiler errors.       */
/*  d26352  20180810  Compiler compatibility updates                 */
/*  d26048  20190115  XWB support                                    */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/*********************************************************************/
#ifndef _SLECB_H
#define _SLECB_H 1

#include <tpf/tpfeq.h>
#include <tpf/c_idspat.h>
#include <tpf/c_idsuxt.h>
#include <tpf/tpfapi.h>
#include <locale.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <tpf/sysapi.h>
#include <tpf/c_ck1ke.h>
#include <tpf/c_ilvl.h>



#define C_PGM  CINFC_CMMCTK2
#define C_PGM1 CINFC_CMMDFSW
#define MAINTASKL  "Lodicdrv"
#define MAINTASKI  "Inputdrv"
#define MAINDRIVER  "maindriv"

#define RESOURCE_MAGICNUMBER  30
#define ITERATION_MAGICNUMBER 2

#define YES 'Y'
#define NO  '\0'
#define INLSHUTD 'L'
#define STARTLOADING 'S'
#define LODICSHUTDOWNLEVEL     'L'
#define INPUTLISTSHUTDOWNLEVEL 'I'

/* List name */

#define NOLIST        0
#define READYDLIST    1
#define INPUTLIST     2
#define SHUTDOWNLIST  3
#define DEFERLIST     4
#define STNDBY        5

/* resource name */

#define SYSHEAP 0
#define SWB     1
#define ECB     2
#define FRM     3
#define CMB     4
#define _1MGFRM 5
#define XWB     6                                                                        //d26048



/* OutPut MMSG */

#define INFO_MSGLI "LODIC SHUTDOWN WAS DETECTED"
#define INFO_MSGL  "LODIC IS ABOVE LIMIT"
#define INFO_MSGI  "INPUT LIST SHUTDOWN ABOVE LIMIT"
#define INFO_SHD   "INPUT LIST SHUTDOWN WAS DETECTED"

#define INFO_RAL   "ATTEMPTING TO PUT LODIC LEVEL BELOW RESOURCE=("
#define INFO_RAI   "ATTEMPTING TO PUT INPUT LIST LEVEL BELOW RESOURCE=("

#define INFO_AGAIN "RELOADING REQUESTORS..."
#define INFO_DS    "STARTING DRIVER ... "
#define INFO_DD    "DONE TESTING SHUTDOWN ..."
#define INFO_RSI   "ISHUT - RELEASING-"
#define INFO_RSL   "LODIC - RELEASING-"
#define INFO_SR    "DONE RETURNING RESOURCE: "
#define ERR_NOHEAP "ERROR: NO HEAP TABLE AVAILABLE"
#define INFO_ECBR  "ALREADY RUNNING"
#define INFO_HTF   "HEAP TABLE FULL"
#define INFO_NHEAP "NO MEMORY TO ALLOCATE HEAP"
#define CARVING    "Carving completed"
int printed_lodicshutdown = 0;

/* For Printing */
char *type_msg[7] = {NULL, "SWBs","ECBs","FRMs","CMBs","FRM1MB","XWBs"};                 //d26048

/* Last LFRAME is a place holder for 1 meg frame.  */

enum p_blktype numblk[7] = {LMAX,LSWB,LECB,LFRAME,LCOMMON, LFRM1MB,LXWB};                //d26048

/* More to come later */

#define MaxChilds 2000 /* Total number of ECB */

#define TOTAL_OP 7                                                                       //d26048




#define MEMORY_PART 300
struct _1kblock
{
   struct _1kblock *nextblock;
   char owner;
   int size;
};




typedef struct struct_task struct_task;

struct struct_task
{
      char pid[9];
      unsigned char list_name;
      unsigned char resource_name;
      short int n_blocks;
      short int pid_id;
      unsigned char running;
};


typedef struct sysinfo sysinfo;

struct sysinfo
{


   char shutdownlist;         /* 'I' Inputlist  shutdown level  */
                              /* 'L' Lodic shutdown level       */
   unsigned char shutdown;    /* Flag to signal shutdown state  */


   struct_task  ecb[MaxChilds]; /* Structure for data structure */
   int ecb_counter;

   short int msg_flag;
   char request_exit;

   int flag_tset;
   int timer;
   int flag_sset;
   int rsize;
   int flag_rset;
   unsigned char rra;         /* Random Resource Allocator      */
   unsigned char _argc;
   int ecb_no;
   long swb_addr;
   char time_in[16];
   char r_sd[8];
   unsigned char delay;
};

#define  FRAME_SIZE ((sizeof(struct sysinfo)/4096)+1)

struct lod_ic
{
    int ecb;
    int iob;
    int frm;
    int cmb;
    int swb;
    int frm1mb;	/* for one meg frame */
    int us1;         /* User area 1 */                                                   //d26048
    int us2;         /* User area 2 */                                                   //d26048
    int xwb;                                                                             //d26048
    int reserved_fullwords[7];                                                           //d26048
};

int   cmb_rsize = 30;
int   frm_rsize = 30;
int   swb_rsize = 30;
int   xwb_rsize = 30;                                                                    //d26048
int   ecb_rsize = 1;
int   fr1mb_rsize = 1;



/***************************/
/***************************/
/* FOR OWNER COMPUTATIONS  */
/***************************/
/***************************/



#define QXN1RUNS "QXN14KFR"
struct parameters
{

   long  api;
   void * p1;  /* First */
   long p2;  /* Second */
   void * p3;  /* Third */
   long p4;  /* Fourth */
   void * p5;  /* Fifth */
   unsigned char funct;

};
struct owner
{
  char hile[TOTAL_OP][9];
  char mile[TOTAL_OP][9];
  char lole[TOTAL_OP][17];
  int  size[TOTAL_OP];
  char api[TOTAL_OP]; /* C=C function or M=Macro version */
  char action; /* Get/Release/Display/Stop*/
  int blktype[TOTAL_OP];
  char gsyscname[9];
  int idname;
  char def;

};

struct _1kblock * swb_blocks    = NULL;  /* For swb */
struct _1kblock * xwb_blocks    = NULL;  /* For xwb */                                   //d26048
struct _1kblock * frm_blocks    = NULL;  /* For 4k FRM */
struct _1kblock * frm1mb_blocks  = NULL;
struct _1kblock * cmb_blocks    = NULL;
struct _1kblock * sh_blocks     = NULL;



int printinfo(char *s)
{
   struct wtopc_header  message_header;
   wtopc_insert_header(&message_header, "LISHD", 51,
            'I', WTOPC_SYS_TIME);



     wtopc(s, WTOPC_EBROUT+WTOPC_PRC, WTOPC_NO_CHAIN,&message_header);
     return 0;
}

int event_handler(struct ev0bk *event,char *event_name, int timer)
{
	
	

	event->evnpstinf.evnbkc1 = 1;

	if (event_name == NULL)
    {
		evntc(event, EVENT_CNT,'N', timer, EVNTC_NORM);
		evnwc(event, EVENT_CNT);
	}
	
	else
	{
		memcpy((char *)event->evnbkn, event_name,8);
        evntc(event, EVENT_CNT, 'Y', 0, EVNTC_NORM);

	}

	return 0;
}

int INPUTLIST_ok()
{
  int shutdown;
  tpf_ilsdc(shutdown);
  return shutdown;
}
/* rc = 0 = shutdown */
int inputlist_ok(sysinfo *swb_addr, char *t_rs)
{

    /* Need read access to retrive the values from zdclv table */
    int rc = 0;
    struct idslvl * zdclv = (struct idslvl *) cinfc_fast(CINFC_CMMLVL);

    strcpy(t_rs,"SWB");
    if (zdclv->lvlmebi  > numbc(LECB) )
         strcpy(t_rs,"ECB");
    else if(zdclv->lvlmxwi > numbc(LXWB))                                                //d26048
           strcpy(t_rs,"XWB");                                                           //d26048
    else if  (zdclv->lvlmfri > numbc(LFRAME))
           strcpy(t_rs,"FRM");
    else if(zdclv->lvlmswi > numbc(LSWB))
           strcpy(t_rs,"SWB");
    else if (zdclv->lvlmcmi > numbc(LCOMMON))
            strcpy(t_rs,"CMB");
    else if (zdclv->lvlmfr1mbi > numbc(LFRM1MB))
    strcpy(t_rs,"FRM1MB");
    else rc = 1;

    return rc;

}

int calculate_entries()
{
    struct idslvl * zdclv = (struct idslvl *) cinfc_fast(CINFC_CMMLVL);


    if (zdclv->lvlmfri > MEMORY_PART)
      frm_rsize = zdclv->lvlmfri/MEMORY_PART;

    if (zdclv->lvlmswi > MEMORY_PART)
      swb_rsize = zdclv->lvlmswi/MEMORY_PART;

    if (zdclv->lvlmcmi > MEMORY_PART)
      cmb_rsize = zdclv->lvlmcmi/MEMORY_PART;
    return 0;                                                                   //d26352
}

int check_lodic_values()   {
	struct lod_ic *lod;
	lod  = ( struct lod_ic*)(cinfc_fast_ss(CINFC_CMMLODIC, ecbptr()->ce1dbi));

	if  (numbc(LFRAME) < lod->frm   ||  numbc(LECB) < lod->ecb  ||
		 numbc(LCOMMON) < lod->cmb  ||   numbc(LSWB) < lod->swb ||
         numbc(LXWB) < lod->xwb || numbc(LFRM1MB) < lod->frm1mb)                         //d26048
	{
		return 0;

	}
	return 1;
}






int get_reqsize(int type)
{

      if (type == SWB)
         return swb_rsize;

      if (type == XWB)                                                                   //d26048
         return xwb_rsize;                                                               //d26048
         
      if (type == FRM)
         return frm_rsize;

      if (type == CMB)
         return cmb_rsize;

      return 1; /* For 1 meg frm and ecb */

}

/* Return 0 if we are running low in resource */
int lodic_ok()
{


   int rc;

    /* Check/Get Rc: What kind of resource we want to monitor */
    rc =  lodic_ext((enum t_lodic_opt)(LODIC_ECBCREATE+LODIC_BATCH+LODIC_HOLD),
                     NULL);

   return rc;

}

/***************************/
/* MAXSUSP test cases      */
/***************************/
#include <errno.h>

#define  NUM_1MEG            1
#define  MIN_MAXSUSP         2
#define  NUM_EXTRA           5                                                           //d11295
#define  MSG_SIZE            255
#define  CASE_ID             "MAXSUSP/CRECLASS"
#define  LISH_GSYSC_TOKEN    "LISH_TBL"
#define  FRM1MB_TOKEN        "FRM1MBTK"
#define  FRM_TOKEN           "FRMTOKEN"
#define  MAXSUSP_EVENT       "maxsusp"

#define  MAXSUSP             1
#define  CRECLASS            2

#define  BATCH               0
#define  LOBATCH             1
#define  IBMHI               2
#define  USRLOD1             4
#define  USRLOD2             5
#define  USRLOD3             6
#define  USRLOD4             7

char err_buf[100];                                                              //d26352
#define prterr(x) sprintf(err_buf, "%s ** Case Failed ** \n", x); \
                  wtopc(err_buf, WTOPC_EBROUT+WTOPC_PRC, \
                        WTOPC_NO_CHAIN, WTOPC_NO_HEADER);                       //d26352
#define prtmsg(x) if ( ecbptr()->ebcm01 == TRUE ) \
        wtopc(x, WTOPC_EBROUT+WTOPC_PRC\
               , WTOPC_NO_CHAIN\
               , WTOPC_NO_HEADER); else


/* Struct for mapping parameters passed */
typedef struct {
   char             case_id[sizeof(CASE_ID)];
   unsigned short   resource_type;
   unsigned short   minute_maxsusp;
   short            cre_class;     // CRECLASS
   enum t_lodic_opt enum_cr_class; // enum value of CRECLASS
   char             is_ECBcase;    // Flag for ECBs created by the ECB test case
   char             is_1000FrameCase;  // Flag for ECBs created by Frm test case
   char             printmsg;      // NOVerbose flag
} PARA_PASSED;

void doTestCase(void);
void releaseResource(int, int, TPF_DECB *, long*);
void consumeResource(int, int, TPF_DECB *, long*, PARA_PASSED*);

extern "C" {                       // Get rid of function name mingle
void lodicMacro(void);
}

char msg[MSG_SIZE],
     block_type[7],
     parent_class[8],
     child_class[8],
     *priority_class[] = { "BATCH",
                           "LOBATCH",
                           "IBMHI",
                           "USRLOD1",
                           "USRLOD2",
                           "USRLOD3",
                           "USRLOD4"  },
    *blockTypes[7] =     { NULL,
                           "SWB",
                           "ECB",
                           "4KB FRAME",
                           "CMB",
                           "1MB FRAME"
                           "XWB",};                                                 //d26048

enum t_lodic_opt  lodic_options, cr_option, cr_class;
int  num_maxsusp, isAboveLODIC;
short child_priority;
char is_CRclass = FALSE;

/*********************************************/
/* Return the number of the available blocks */
/*********************************************/
int available_num(unsigned short type)
{
   int  num=0;
   switch (type)
   {
      case SWB:
         num = numbc(LSWB);
         break;
         
      case XWB:                                                                          //d26048
         num = numbc(LXWB);                                                              //d26048
         break;                                                                          //d26048

      case CMB:
         num = numbc(LCOMMON);
         break;

      case FRM:
         num = numbc(LFRAME);
         break;

      case _1MGFRM:
         num = numbc(LFRM1MB);
         break;

      case ECB:
         num = numbc(LECB);
         break;
   }
   return num;
}

/**********************************************************/
/* Return the value set for the LODIC for specified class */
/**********************************************************/
int lodic_num(unsigned short type, int priority_class)
{
   int             num=0;
   struct lod_ic*  lodic_ptr;

   lodic_ptr = (struct lod_ic*) cinfc_fast_ss(CINFC_CMMLODIC, ecbptr()->ce1dbi);

   // Increment the address to the proper priority class (32 bytes each time)
   lodic_ptr += priority_class;

   switch (type)
   {
      case SWB:
         num = lodic_ptr->swb;
         break;

      case XWB:                                                                          //d26048
         num = lodic_ptr->xwb;                                                           //d26048
         break;                                                                          //d26048
         
      case CMB:
         num = lodic_ptr->cmb;
         break;

      case FRM:
         num = lodic_ptr->frm;
         break;

      case _1MGFRM:
         num = lodic_ptr->frm1mb;
         break;

      case ECB:
         num = lodic_ptr->ecb;
         break;
   }

   return num;
}

int inputlist_num(unsigned short type)                                                   //d11295
{
   int             num=0;                                                                //d11295

   /* Need read access to retrive the values from zdclv table */
   struct idslvl * zdclv = (struct idslvl *) cinfc_fast(CINFC_CMMLVL);                   //d11295

   switch (type)                                                                         //d11295
   {
      case SWB:                                                                          //d11295
         num = zdclv->lvlmswi;                                                           //d11295
         break;                                                                          //d11295

      case CMB:                                                                          //d11295
         num = zdclv->lvlmcmi;                                                           //d11295
         break;                                                                          //d11295

      case FRM:                                                                          //d11295
         num = zdclv->lvlmfri;                                                           //d11295
         break;                                                                          //d11295

      case _1MGFRM:                                                                      //d11295
         num = zdclv->lvlmfr1mbi;                                                        //d11295
         break;                                                                          //d11295

      case ECB:                                                                          //d11295
         num = zdclv->lvlmebi;                                                           //d11295
         break;                                                                          //d11295
   }

   return num;                                                                           //d11295
}                                                                                        //d11295

#endif
